'******************************************************************************
'*  yaz80.spin - Yet another Z80 emulator - using the LMM technique
'*
'*  (c) Juergen Buchmueller <pullmoll@t-online.de>
'*
'* $Id: yaz80.spin,v 1.31 2010-03-26 14:27:00 pm Exp $
'******************************************************************************
CON

	' Z80 flag registers
	CF			=	%00000001	' carry
	NF			=	%00000010	' negative
	PF			=	%00000100	' parity (also overflow)
	VF			=	PF
	XF			=	%00001000	' undocumented x
	HF			=	%00010000	' auxiliary (half) carry
	YF			=	%00100000	' undocumented y
	ZF			=	%01000000	' zero
	SF			=	%10000000	' sign

	' just handy names for the constants
	bit0			=	%00000001
	bit1			=	%00000010
	bit2			=	%00000100
	bit3			=	%00001000
	bit4			=	%00010000
	bit5			=	%00100000
	bit6			=	%01000000
	bit7			=	%10000000

	IFF1			=	%00000001
	IFF2			=	%00000010
	IM1			=	%00000100
	IM2			=	%00001000

	io_cmd_out		=	$01		' command to write to an i/o port
	io_cmd_in		=	$02		' command to read from an i/o port
	io_cmd_break		=	$03		' command to execute a break (halt)
	io_cmd_kbd		=	$04		' command to read the TRS-80 keyboard matrix

	constat_port		=	$10		' console status port
	condata_port		=	$11		' console data port

VAR
	long	cog
#ifdef INTERRUPTS
	long	irq
#endif INTERRUPTS

PUB start(ram, io, screen, cycles) : okay

'' Start CPU simulator - starts a cog
'' returns false if no cog available
''
'' ram is a pointer to Z80 RAM space
'' io is a pointer to Z80 I/O space

	stop
	io_command := io
	io_port := io + 4
	io_data := io + 8

#ifdef TRS80
	screen_ptr := screen - $3c00
#endif TRS80

#ifdef INTERRUPTS
	int_request := @irq
	irq := 0
#endif

#ifdef COUNT_C
	cc_ptr := cycles
	tm_ptr := cycles + 4
#endif

	okay := cog := COGNEW(@entry, ram) + 1

PUB stop
'' Stop CPU simulator - frees a cog
	if cog
		cogstop(cog~ - 1)

#ifdef INTERRUPTS
PUB get_irq
	return @irq
#endif

DAT
		org	0
entry

#ifdef TriBladeProp
'The TriBladeProp Blade #2 driver presets the latch to enable SRAM -CE(U23)=0 and hold LE=0
datax		mov     outa, ram_latched		' -WE=1 -OE=1, all other bits=0
outx		mov     dira, ram_dir_read		' setup direction to read
		jmp	#startup			' contains hub RAM address for ea (overwrites jmp)
#endif TriBladeProp

#ifdef RamBladeProp
'The RamBladeProp needs this?
datax		mov     outa,#0
outx		jmp	#startup
#endif RamBladeProp

#ifdef DracBladeProp
outx		jmp	#startup			' contains address masks for ram_address setup (overwrites jmp)
#endif DracBladeProp

#ifdef PropRPM
hubptr		jmp	#startup
#endif PropRPM

ram_base	long	0-0				' base address of the Z80 RAM in hub RAM
io_command	long    0-0				' io command address
io_port		long    0-0				' io port number
io_data		long    0-0				' io port data

table_00	long	@@@opcodes_00
table_cb	long	@@@opcodes_cb
table_ed	long	@@@opcodes_ed
table_xy	long	@@@opcodes_xy
table_xy_cb	long	@@@opcodes_xy_cb

#ifdef TRS80
screen_ptr	long	0-0				' address of the 1K screen buffer in hub RAM
rd_memory_ptr	long	@@@rd_memory			' dispatch table for read memory handlers
wr_memory_ptr	long	@@@wr_memory			' dispatch table for write memory handlers
#endif

#ifdef INTERRUPTS
int_request	long	0-0
int_handler	long	@@@do_int
#endif

regram_ptr	long	@@@C_reg			' C register address
IX		long	@@@IX_reg			' IX register address
IY		long	@@@IY_reg			' IY register address
I		long	@@@I_reg			' interrupt vector register address
low_word	long	$0000ffff

C		long	0
B		long	0
E		long	0
D		long	0
L		long	0
H		long	0
F		long	0
A		long	0
R		long	0				' refresh register
R2		long	0				' bit #7 of last write to R
IFF		long	0				' interrupt flip-flops
SP		long	0				' stack pointer
PC		long	0				' program counter
XY		long	0

alu		long	0				' arithmetic logic unit (sort of :)
aux		long	0				' auxiliary flag temp
tmp		long	0				' second operand temp
ea		long	0				' effective address
daa_adj		long					' adjustment value for DAA (reused)
t1		long	0				' temporary 1

lmm_pc		long	0				' LMM program counter

#ifdef COUNT_C
cc_ptr		long	0-0
tm_ptr		long	0-0
cc		long	0
tm		long	0
d1_000_000	long	1_000_000
#endif

#ifdef DEBUG
char		long	0
#endif DEBUG 

startup
		mov	ram_base, PAR
		mov	PC, #0
		mov	F, #ZF

#ifdef COUNT_C
measure
		mov	cc, d1_000_000
		mov	tm, CNT
#endif COUNT_C

'******************************************************************************
'
' Fetch the next opcode from RAM[PC++] and dispatch
'
fetch
#ifdef COUNT_C
		cmps	cc, #0			WZ, WC	' cycle count underflow?
	if_a	jmp	#:skip
		sub	cc, d1_000_000
		abs	cc, cc
		wrlong	cc, cc_ptr
		sub	tm, CNT
		abs	tm, tm
		wrlong	tm, tm_ptr
		jmp	#measure
:skip
#endif COUNT_C

#ifdef DEBUG
		call	#k_break
#endif DEBUG

#ifdef INTERRUPTS
		rdlong	tmp, int_request	WZ
	if_nz	mov	lmm_pc, int_handler
	if_nz	jmp	#lmm_next
fetch0
#endif INTERRUPTS

		mov	tmp, table_00
fetch1
#ifdef COUNT_R
		add	R, #1
#endif COUNT_R
		call	#rd_opcode
		add	tmp, alu
		add	tmp, alu
fetch2
		rdword	lmm_pc, tmp
lmm_next
		rdlong	:op1, lmm_pc
		add	lmm_pc, #4
:op1		nop
		rdlong	:op2, lmm_pc
		add	lmm_pc, #4
:op2		nop
		rdlong	:op3, lmm_pc
		add	lmm_pc, #4
:op3		nop
		rdlong	:op4, lmm_pc
		add	lmm_pc, #4
:op4		nop
		jmp	#lmm_next

'******************************************************************************
'
'	Memory access functions (TriBladeProp)
'
'******************************************************************************
#ifdef TriBladeProp
'                                          +-------- -CS (U26 FLASH)   -+                                  
'                                          |+------- -CS (J22 microSD)  +- Latched pins...                 
'                                          ||+------ -RST3              |    passes when LE=1              
'             (P30) SO ---+                |||+----- -RST1              |    latches on  LE low going edge 
'             (P31) SI --+|                ||||+---- -CE (U24 SRAM-hi)  |    latched on  LE=0              
'                        ||+---- -WE(SDA)  |||||+--- -CE (U23 SRAM-lo)  |                                  
'                        |||+--- -OE(SCL)  ||||||+-- A20                |                                  
'                        ||||+--  LE       |||||||+- A19               -+                                  
'                        |||||             ||||||||                                                        
'          P31...P0 --> %00xx0_xxxxxxxxxxx_xxxxxxxx_xxxxxxxx  
ram_latched	long	%00110_00000000000_00000000_00000000	' -WE=1 -OE=1 (LE=0)                               
ram_read	long	%00100_00000000000_00000000_00000000	' -WE=1 -OE=0 (LE=0)    
ram_write	long	%00010_00000000000_00000000_00000000	' -WE=0 -OE=1 (LE=0)
ram_dir_read	long	%00110_11111111111_11111111_00000000	' outputs WE, OE, A0-18, inputs D0-7
ram_dir_write	long	%00110_11111111111_11111111_11111111	' outputs WE, OE, A0-18, D0-7
ram_dir_input	long	%00000_00000000000_00000000_00000000	' all inputs (for Ram Disk access)
'                              +---------+ +------+ +------+
'                              A18......A8 A7....A0 D7....D0
'                                          Q7....Q0
'

'*******************************************************
' Read a byte from RAM[HL] into the alu
'
rd_byte_hl
		mov	ea, H
		shl	ea, #8
		or	ea, L
		' fall through
'*******************************************************
' Read a byte from RAM[ea] into the alu
'
rd_byte
		and	ea, low_word
#ifdef TRS80
		mov	outx, ea
		shr	outx, #9			' discard 9 bits
		and	outx, #%01111110		' discard 1 more bits
		add	outx, rd_memory_ptr		' add pointer to read memory table
		rdword	outx, outx			' fetch the pointer
		jmp	outx				' and jump there
rd_rom
rd_ram
#endif TRS80
		mov	outx, ea
		shl	outx, #8			' shift address 8 bits
		or	outx, ram_read			' add -WE=1 -OE=0
		mov	outa, outx			' pc (shifted 8 bits + -WE=1 -OE=0)
		nop					' delay for sram access time
		mov     alu, ina			' read SRAM
		and     alu, #$ff			' extract 8 bits
rd_byte_ret
rd_byte_hl_ret
		ret

#ifdef TRS80
rd_nop
		mov	alu, #$00
		jmp	#rd_byte_ret
rd_scr
		mov	outx, ea
		add	outx, screen_ptr
		rdbyte	alu, outx
		jmp	#rd_byte_ret
rd_i_o
		mov	alu, #$ff			' not yet implemented
		jmp	#rd_byte_ret
rd_kbd
		mov	io_port, ea
		mov	t1, #io_cmd_kbd
		wrlong	t1, io_command
:wait		rdlong	t1, io_command		WZ
	if_z	jmp	#:wait
		rdbyte	alu, io_data
		jmp	#rd_byte_ret
#endif TRS80

'*******************************************************
' Write a byte from the alu into RAM[DE]
'
wr_byte_de
		mov	ea, D
		shl	ea, #8
		or	ea, E
		' fall through
'*******************************************************
' Write a byte from the alu into RAM[ea]
'
wr_byte
		and	ea, low_word
#ifdef TRS80
		mov	outx, ea
		shr	outx, #9			' discard 9 bits
		and	outx, #%01111110		' discard 1 more bits
		add	outx, wr_memory_ptr		' add pointer to write memory table
		rdword	outx, outx			' fetch the pointer
		jmp	outx				' and jump there
wr_ram
#endif TRS80
		mov	outx, ea
		shr	outx, #8		
		mov	outx, ea
		shl	outx, #8			' shift address 8 bits
		or	outx, ram_write			' add -WE=0 -OE=1
		mov	datax, alu
		and	datax, #$ff			' ensure upper bytes=0
		or	outx, datax			' add data bits to be written
		mov	outa, outx			' output -WE=0 -OE=1, address bits, data bits
		mov	dira, ram_dir_write		' set data bits to outputs
		or      outa, ram_latched		' set -WE=0->1 -OE=1->1  (end write)
		mov     dira, ram_dir_read		' set data bits to inputs
		mov     outa, ram_read			' set -WE=1 -OE=0
#ifdef TRS80
wr_rom
wr_nop
#endif
wr_byte_ret
wr_byte_de_ret
		ret

#ifdef TRS80
wr_i_o
		jmp	#wr_byte_ret			' not yet

wr_scr
		mov	t1, alu
		cmp	t1, #$20		WC
	if_c	or	t1, #$40
		mov	outx, ea
		add	t1, screen_ptr
		wrbyte	alu, outx
		jmp	#wr_byte_ret
#endif TRS80
#endif TriBladeProp





'******************************************************************************
'
'	Memory access functions (RamBladeProp)
'
'******************************************************************************
#ifdef RamBladeProp
' RamBlade Hardware Access:
'       P0-18  = A0-18   SRAM address
'       P19    = -CS     microSD
'       P20    = -CE     SRAM & enable eeprom
'       P21    = -WE     SRAM & enable eeprom
'       P22    = SO      \ not used by this driver
'       P23    = SI      /
'       P24-31 = D0-7    SRAM \ * see below
'                             | P28 = SCL eeprom
'                             | P29 = SDA eeprom
'                             | P30 = prog SO
'                             / P31 = prog SI
'
'         (microSD) DO ---------+
'         (microSD) DI --------+| +--------  SI (P23)
'         (microSD) CLK-------+|| |+-------  SO (P22)
'                             ||| ||
'             (P28) SCL ----+ ||| ||+------ -WE (SRAM & enable eeprom)
'             (P29) SDA ---+| ||| |||+----- -CE (SRAM & enable eeprom)
'             (P30) SO ---+|| ||| ||||+---- -CS (microSD)
'             (P31) SI --+||| ||| |||||
'                        |||| ||| |||||
'          P31...P0 --> %xxxxxxxx_00xxx_xxxxxxxxxxxxxxxxxxx
'       10K pullups -->    ** ***   ***
'                        +------+       +-----------------+
'                        D7....D0       A18..............A0
ram_dir_read    long    %00000000_00010_1111111111111111111 ' outputs CE, A0-18           (sram read)
ram_dir_write   long    %11111111_00000_1111111111111111111 ' outputs A0-18, D0-7         (sram disabled)
ram_dir_wstrobe long    %11111111_00110_1111111111111111111 ' outputs CE, WE, A0-19, D0-7 (write strobe)
'
'*******************************************************
' Read a byte from RAM[HL] into the alu
'
rd_byte_hl
		mov	ea, H
		shl	ea, #8
		or	ea, L
		' fall through
'*******************************************************
' Read a byte from RAM[ea] into the alu
'
rd_byte
		and	ea, low_word
#ifdef TRS80
		mov	outx, ea
		shr	outx, #9			' discard 9 bits
		and	outx, #%01111110		' discard 1 more bits
		add	outx, rd_memory_ptr		' add pointer to read memory table
		rdword	outx, outx			' fetch the pointer
		jmp	outx				' and jump there
rd_rom
rd_ram
#endif
		mov	outa, ea			' apply the read address incl -CE=0 -WE=0
		mov	dira, ram_dir_read		' set outputs A0-18, CE
		mov	alu, ina			' read byte from SRAM
		shr	alu, #24			' bits 31..24 to 7..0
rd_byte_ret
rd_byte_hl_ret
		ret

#ifdef TRS80
rd_nop
		mov	alu, #$00
		jmp	#rd_byte_ret
rd_scr
		mov	outx, ea
		add	outx, screen_ptr
		rdbyte	alu, outx
		jmp	#rd_byte_ret
rd_i_o
		mov	alu, #$ff			' not yet implemented
		jmp	#rd_byte_ret
rd_kbd
		mov	io_port, ea
		mov	t1, #io_cmd_kbd
		wrlong	t1, io_command
:wait		rdlong	t1, io_command		WZ
	if_z	jmp	#:wait
		rdbyte	alu, io_data
		jmp	#rd_byte_ret
#endif TRS80

'*******************************************************
' Write a byte from the alu into RAM[DE]
'
wr_byte_de
		mov	ea, D
		shl	ea, #8
		or	ea, E
		' fall through
'*******************************************************
' Write a byte from the alu into RAM[ea]
'
wr_byte
		and	ea, low_word
#ifdef TRS80
		mov	outx, ea
		shr	outx, #9			' discard 9 bits
		and	outx, #%01111110		' discard 1 more bits
		add	outx, wr_memory_ptr		' add pointer to write memory table
		rdword	outx, outx			' fetch the pointer
		jmp	outx				' and jump there
wr_ram
#endif TRS80
		mov	outa, ea			' apply the read address incl -CE=0 -WE=0
		mov	dira, ram_dir_write		' set outputs A0-18, CE
		mov	outx, alu
		rol	outx, #24			' data byte in bits 31..24
		or	outx, ea			' address in bits 18..0
		mov	outa, outx
		mov	dira, ram_dir_wstrobe		' -CE=-WE= 1 -> 0       \ ce + we pulse
		nop					' sram delay            |
		mov	dira, ram_dir_write		' address + data only   /
#ifdef TRS80
wr_rom
wr_nop
#endif
wr_byte_ret
wr_byte_de_ret
		ret
#ifdef TRS80
wr_i_o
		jmp	#wr_byte_ret			' not yet

wr_scr
		mov	t1, alu
		cmp	t1, #$20		WC
	if_c	or	t1, #$40
		mov	outx, ea
		add	outx, screen_ptr
		wrbyte	t1, outx
		jmp	#wr_byte_ret
#endif TRS80

#endif RamBladeProp





'******************************************************************************
'
'	Memory access functions (DracBladeProp)
'
'******************************************************************************
#ifdef DracBladeProp
'****************** Dr_Acula single propeller driver **************************
' Three latches to access the ram chip - see main program for spin version
' uses variables 'ea' and 'alu'. skip banked memory and checking for out of
' bounds address will need two nop instructions in /rd and /wr to be sure 
'
dir_138		long    %00000000_00000000_00001111_00000000	' for reads so data lines are tristate till the read
address_low	long    %00000000_00000000_00000100_00000000	' low address latch = xxxx010x and gate low xxxxxxx0
address_mid	long    %00000000_00000000_00000110_00000000	' middle address latch = xxxx011x and gate low xxxxxxx0
write_enable	long    %00000000_00000000_00000010_00000000	' write enable xxxx001x and xxxxxxx0
'
'*******************************************************
' Read a byte from RAM[HL] into the alu
'
rd_byte_hl
		mov	ea, H
		shl	ea, #8
		or	ea, L
		' fall through
'*******************************************************
' Read a byte from RAM[ea] into the alu
'
rd_byte
		and	ea, low_word
#ifdef TRS80
		mov	outx, ea
		shr	outx, #9			' discard 9 bits
		and	outx, #%01111110		' discard 1 more bits
		add	outx, rd_memory_ptr		' add pointer to read memory table
		rdword	outx, outx			' fetch the pointer
		jmp	outx				' and jump there
rd_rom
rd_ram
#endif TRS80
		call	#ram_address			' sets up the latches with the correct ram address
		mov	dira, dir_138			' for reads so P0-P7 tristate till do read
		mov	outa, #$0			' set 000 low 138 read, low gate and P0-P7 are inputs
		nop					' none does not work, 1 does work
		mov	alu, ina			' read SRAM
		and	alu, #$ff			' extract 8 bits
		or	outa, #%1_00000000		' set the gate high again
		mov	dira, #0			' tristate all pins
rd_byte_ret
rd_byte_hl_ret
		ret

#ifdef TRS80
rd_nop
		mov	alu, #$00
		jmp	#rd_byte_ret
rd_scr
		mov	outx, ea
		add	outx, screen_ptr
		rdbyte	alu, outx
		jmp	#rd_byte_ret
rd_i_o
		mov	alu, #$ff			' not yet implemented
		jmp	#rd_byte_ret
rd_kbd
		mov	io_port, ea
		mov	t1, #io_cmd_kbd
		wrlong	t1, io_command
:wait		rdlong	t1, io_command		WZ
	if_z	jmp	#:wait
		rdbyte	alu, io_data
		jmp	#rd_byte_ret
#endif TRS80

'*******************************************************
' Write a byte from the alu into RAM[DE]
'
wr_byte_de
		mov	ea, D
		shl	ea, #8
		or	ea, E
		' fall through
'*******************************************************
' Write a byte from the alu into RAM[ea]
'
wr_byte
		and	ea, low_word
#ifdef TRS80
		mov	outx, ea
		shr	outx, #9			' discard 9 bits
		and	outx, #%01111110		' discard 1 more bits
		add	outx, wr_memory_ptr		' add pointer to write memory table
		rdword	outx, outx			' fetch the pointer
		jmp	outx				' and jump there
wr_ram
#endif TRS80
        	call	#ram_address			' sets up the latches with the correct ram address
        	mov	outx, alu			' get the byte to output
        	and	outx, #$ff			' ensure upper bytes=0
        	or	outx, write_enable		' or with correct 138 address which is the same as latchdirection2
        	mov	outa, outx			' send it out
        	nop					' NOP  needs at least one due to 138 propogation delay
        	or	outa, #%1_00000000		' set it high again
        	mov	dira, #0			' tristate all pins
#ifdef TRS80
wr_rom
wr_nop
#endif
wr_byte_ret
wr_byte_de_ret
		ret

#ifdef TRS80
wr_i_o
		jmp	#wr_byte_ret			' not yet

wr_scr
		mov	t1, alu
		cmp	t1, #$20		WC
	if_c	or	t1, #$40
		mov	outx, ea
		add	outx, screen_ptr
		wrbyte	t1, outx
		jmp	#wr_byte_ret
#endif TRS80

'*******************************************************
' Sets up the ram latches.
' Assumes high latch A16-A18 low so only
' accesses 64K of RAM
ram_address
		mov	dira, dir_138			' set up the pins for programming latch chips
		or	dira, #$ff			' set P0-P7 high
		mov	outx, ea			' get the address into a temp variable
		and	outx, #$ff			' mask the low byte
		or	outx, address_low		' or with 138 low address
		mov	outa, outx			' send it out
		or	outa, #%1_00000000		' set it high again and repeat for the middle byte
		mov	outx, ea			' get the address into a temp variable
		shr	outx, #8			' shift right by 8 places
		or	outx, address_mid		' or with 138 middle address
		mov	outa, outx			' send it out
		or	outa, #%1_00000000		' set it high again
ram_address_ret
		ret
#endif DracBladeProp





#ifdef PropRPM
'******************************************************************************
'
'	Memory access functions (PropRPM)
'
'******************************************************************************
'*******************************************************
' Read a byte from RAM[HL] into the alu
'
rd_byte_hl
		mov	ea, H
		shl	ea, #8
		or	ea, L
		' fall through
'*******************************************************
' Read a byte from RAM[ea] into the alu
'
rd_byte
		and	ea, low_word
		mov	hubptr, ea
		add	hubptr, ram_base
		rdbyte	alu, hubptr
rd_byte_ret
rd_byte_hl_ret
		ret

'*******************************************************
' Write a byte from the alu into RAM[DE]
'
wr_byte_de
		mov	ea, D
		shl	ea, #8
		or	ea, E
		' fall through
'*******************************************************
' Write a byte from the alu into RAM[ea]
'
wr_byte
		and	ea, low_word
		mov	hubptr, ea
		add	hubptr, ram_base
		wrbyte	alu, hubptr
wr_byte_ret
wr_byte_de_ret
		ret
#endif PropRPM




'******************************************************************************
'
' Read a byte from address [PC++] into alu
'
rd_opcode
		mov	ea, PC
		add	PC, #1
		and	PC, low_word
		call	#rd_byte
rd_opcode_ret
		ret

'******************************************************************************
'
' Read a word from address [PC++] into alu
'
rd_opword
		mov	ea, PC
		add	PC, #2
		and	PC, low_word
'******************************************************************************
'
' Read a word from address [ea] into alu
'
' clobbers: t1
'
rd_word
		call	#rd_byte
		mov	t1, alu
		add	ea, #1
		call	#rd_byte
		shl	alu, #8
		or	alu, t1
		sub	ea, #1
rd_word_ret
rd_opword_ret
		ret

'******************************************************************************
'
' Write a word from alu to address [ea]
'
wr_word
		call	#wr_byte
		add	ea, #1
		ror	alu, #8
		call	#wr_byte
		rol	alu, #8
		sub	ea, #1
wr_word_ret
		ret

'*******************************************************
' Write a byte from the alu to I/O port [BC]
'
wr_port_bc
		mov	ea, B
		shl	ea, #8
		or	ea, C
		' fall through
'*******************************************************
' Write a byte from the alu to an I/O port
'
wr_port
		wrword	ea, io_port
		wrbyte	alu, io_data
#ifdef TriBladeProp
		mov     dira, ram_dir_input		' pass I/O to another cog
#endif TriBladeProp
		mov	t1, #io_cmd_out
		wrbyte	t1, io_command
:wait		rdbyte	t1, io_command		WZ	' ...and wait for it to be completed
	if_nz	jmp	#:wait
#ifdef TriBladeProp
		mov     dira, ram_dir_read		' take I/O from another cog
#endif TriBladeProp
wr_port_ret
wr_port_bc_ret
		ret

'*******************************************************
' Read a byte from I/O port [BC] into the alu
'
rd_port_bc
		mov	ea, B
		shl	ea, #8
		or	ea, C
		' fall through
'*******************************************************
' Read a byte from an I/O port into the alu
'
rd_port
		wrword	ea, io_port
#ifdef TriBladeProp
		mov     dira, ram_dir_input		' pass I/O to another cog
#endif TriBladeProp
		mov	t1, #io_cmd_in
		wrbyte	t1, io_command
:wait		rdbyte	t1, io_command		WZ	'...and wait for it to be completed.
	if_nz	jmp	#:wait
#ifdef TriBladeProp
		mov     dira, ram_dir_read		' take I/O from another cog
#endif TriBladeProp
		rdbyte	alu, io_data
rd_port_ret
rd_port_bc_ret
		ret

'******************************************************************************
'
' Compute an 8 bit increment and set the flags
'
alu_inc
		add	alu, #1				' increment alu
		andn	F, #NF				' reset negative flag
		mov	aux, alu
		and	aux, #15
		cmp	aux, #0		WZ		' lower nibble wrapped to 0?
		muxz	F, #HF				' set half carry
		and	alu, #$FF	WZ		' result is zero?
		muxz	F, #ZF				' Z flag cleared or set
		test	alu, #$80	WZ		' bit #7 set?
		muxnz	F, #SF				' S flag used to indicate sign
		cmp	alu, #$80	WZ		' increment 7f -> 80 ?
		muxz	F, #VF				' V flag used to indicate overflow
#ifdef XYFLAGS
		test	alu, #XF	WZ
		muxnz	F, #XF
		test	alu, #YF	WZ
		muxnz	F, #YF
#endif XYFLAGS
alu_inc_ret
		ret

'******************************************************************************
'
' Compute an 8 bit decrement and set the flags
'
alu_dec
		sub	alu, #1				' decrement alu
		or	F, #NF				' set negative flag
		mov	aux, alu
		and	aux, #15
		cmp	aux, #15	WZ		' lower nibble wrapped to 15?
		muxz	F, #HF				' set half carry
		and	alu, #$ff	WZ		' result is zero?
		muxz	F, #ZF				' Z flag cleared or set
		test	alu, #$80	WZ		' bit #7 set?
		muxnz	F, #SF				' S flag used to indicate sign
		cmp	alu, #$7f	WZ		' decrement 80 -> 7f ?
		muxz	F, #VF				' V flag used to indicate overflow
#ifdef XYFLAGS
		test	alu, #XF	WZ
		muxnz	F, #XF
		test	alu, #YF	WZ
		muxnz	F, #YF
#endif XYFLAGS
alu_dec_ret
		ret

'******************************************************************************
'
' Compute sign, zero, auxiliar, overflow and carry flags after an
' arithmetic instruction
'
szhvc_flags
		xor	aux, tmp
		xor	aux, alu
		test	aux, #HF	WC		' H flag change?
		muxc	F, #HF
		test	aux, #$100	WC
		muxc	F, #(VF | CF)
		test	aux, #$80	WC
	if_c	xor	F, #VF
		test	alu, #$80	WC
		muxc	F, #SF
		and	alu, #$ff	WZ
		muxz	F, #ZF
#ifdef XYFLAGS
		test	alu, #XF	WC
		muxc	F, #XF
		test	alu, #YF	WC
		muxc	F, #YF
#endif XYFLAGS
szhvc_flags_ret
		ret

'******************************************************************************
'
' Compute sign, zero and parity flags after logic/shift/rotate instruction
'
szp_flags_ff
		and	alu, #$ff	WZ, WC		' get zero and parity flags
szp_flags
		muxz	F, #ZF				' if zero set Z flag
		muxnc	F, #PF				' if even parity set the P flag
		test	alu, #$80	WC		' get sign in prop's carry flag
		muxc	F, #SF				' set S flag
#ifdef XYFLAGS
		test	alu, #XF	WC
		muxc	F, #XF
		test	alu, #YF	WC
		muxc	F, #YF
#endif XYFLAGS
szp_flags_ret
szp_flags_ff_ret
		ret

'******************************************************************************
'
' Add tmp to accumulator
'
alu_add
		mov	F, #0				' clear all flags
		mov	alu, A
		mov	aux, alu
		add	alu, tmp			' add the tmp value
		call	#szhvc_flags
		mov	A, alu
		jmp	#fetch

'******************************************************************************
'
' Add tmp with carry to accumulator
'
alu_adc
		test	F, #CF		WC		' get the C flag into carry
		mov	F, #0				' clear all flags
		mov	alu, A
		mov	aux, alu
		addx	alu, tmp			' add the tmp value with carry
		call	#szhvc_flags
		mov	A, alu
		jmp	#fetch

'******************************************************************************
'
' Subtract tmp from accumulator
'
alu_sub
		mov	F, #NF				' clear all flags but the N flag
		mov	alu, A
		mov	aux, alu
		sub	alu, tmp			' subtract the tmp value
		call	#szhvc_flags
		mov	A, alu
		jmp	#fetch

'******************************************************************************
'
' Subtract tmp with carry from accumulator
'
alu_sbc
		test	F, #CF		WC		' get the C flag into carry
		mov	F, #NF				' clear all flags but the N flag
		mov	alu, A
		mov	aux, alu
		subx	alu, tmp			' subtract the tmp value with carry
		call	#szhvc_flags
		mov	A, alu
		jmp	#fetch

'******************************************************************************
'
' Logical and accumulator with tmp
'
alu_and
		mov	F, #HF				' clear all flags but H flag
		mov	alu, A
		and	alu, tmp	WZ, WC		' and with the tmp value
		call	#szp_flags
		mov	A, alu
		jmp	#fetch

'******************************************************************************
'
' Logical exclusive or accumulator with tmp
'
alu_xor
		mov	F, #0				' clear all flags
		mov	alu, A
		xor	alu, tmp	WZ, WC		' and with the tmp value
		call	#szp_flags
		mov	A, alu
		jmp	#fetch

'******************************************************************************
'
' Logical or accumulator with tmp
'
alu_or
		mov	F, #0				' clear all flags
		mov	alu, A
		or	alu, tmp	WZ, WC		' and with the tmp value
		call	#szp_flags
		mov	A, alu
		jmp	#fetch

'******************************************************************************
'
' Compare accumulator with tmp
'
alu_cp
		mov	F, #NF				' clear all flags but the N flag
		mov	alu, A
		mov	aux, alu
		sub	alu, tmp	WZ, WC		' compare with the tmp value
		call	#szhvc_flags
		jmp	#fetch

'******************************************************************************
'
' Add tmp to alu - 16 bit version
'
alu_add16
		and	F, #(SF | ZF | VF)		' keep sign, zero and parity/overflow flags
		mov	aux, alu
		add	alu, tmp
		xor	aux, alu
		xor	aux, tmp
		shr	aux, #8				' get MSB to LSB
		test	aux, #HF	WZ
		muxnz	F, #HF
		mov	aux, alu
		shr	aux, #8
		test	aux, #$100	WZ
		muxnz	F, #CF
alu_add16_ret
		ret

'******************************************************************************
'
' Add tmp to HL - 16 bit version
'
alu_add16_hl
		mov	alu, H
		shl	alu, #8
		or	alu, L
		call	#alu_add16
		mov	L, alu
		mov	H, alu
		and	L, #$ff
		shr	H, #8
		jmp	#fetch

'******************************************************************************
'
' Add tmp to IX/IY - 16 bit version
'
alu_add16_xy
		rdword	alu, XY
		call	#alu_add16
		wrword	alu, XY
		jmp	#fetch

'******************************************************************************
'
' Subtract tmp with carry from accumulator - 16 bit version
'
alu_sbc16
		test	F, #CF		WC		' get C into prop's carry
		subx	L, t1				' subtract the LSB tmp value with carry
		test	L, #$100	WC		' get LSB carry
		and	L, #$ff
		mov	F, #NF				' clear all flags but N flag
		mov	alu, H
		mov	aux, alu
		subx	alu, tmp			' subtract the MSB tmp value with carry
		call	#szhvc_flags
	if_z	cmp	L, #0		WZ
	if_nz	andn	F, #ZF				' clear Z flag if L is non zero
		mov	H, alu
		jmp	#fetch

'******************************************************************************
'
' Add tmp with carry to accumulator - 16 bit version
'
alu_adc16
		test	F, #CF		WC		' get C into prop's carry
		addx	L, t1				' add the LSB tmp value with carry
		test	L, #$100	WC		' get LSB carry
		and	L, #$ff
		mov	F, #0				' clear all flags
		mov	alu, H
		mov	aux, alu
		addx	alu, tmp			' add the MSB tmp value with carry
		call	#szhvc_flags
	if_z	cmp	L, #0		WZ
	if_nz	andn	F, #ZF				' clear Z flag if L is non zero
		mov	H, alu
		jmp	#fetch

'******************************************************************************
'
' Compute rotate left circular and the flags
'
'    alu: [7][6][5][4][3][2][1][0]  carry: [0]
' before   h  g  f  e  d  c  b  a           ?
' after    g  f  e  d  c  b  a  h           h
'
alu_rlc
		mov	F, #0				' clear all flags
		shl	alu, #1
		test	alu, #$100	WC		' get bit #7 into zero flag
		muxc	alu, #$01			' get carry in bit #0
		muxc	F, #CF				' set C flag from carry
		call	#szp_flags_ff
alu_rlc_ret
		ret

'******************************************************************************
'
' Compute rotate right circular and the flags
'
'    alu: [7][6][5][4][3][2][1][0]  carry: [0]
' before   h  g  f  e  d  c  b  a           ?
' after    a  h  g  f  e  d  c  b           a
'
alu_rrc
		mov	F, #0				' clear all flags
		shr	alu, #1		WC
		muxc	alu, #$80			' set bit #7 from carry
		muxc	F, #CF				' set C flag
		call	#szp_flags_ff
alu_rrc_ret
		ret

'******************************************************************************
'
' Compute rotate left through carry and the flags
'
'    alu: [7][6][5][4][3][2][1][0]  carry: [0]
' before   i  h  g  f  e  d  c  b           a
' after    h  g  f  e  d  c  b  a           i
'
alu_rl
		test	F, #CF		WC		' get current C flag in carry
		mov	F, #0				' clear all flags
		rcl	alu, #1				' alu in bits 8..1, carry in bit 0
		test	alu, #$100	WC		' test new carry
		muxc	F, #CF				' set C flag
		call	#szp_flags_ff
alu_rl_ret
		ret

'******************************************************************************
'
' Compute rotate right through carry and the flags
'
'    alu: [7][6][5][4][3][2][1][0]  carry: [0]
' before   i  h  g  f  e  d  c  b           a
' after    a  i  h  g  f  e  d  c           b
'
alu_rr
		test	F, #CF		WZ		' get current C flag in carry
		mov	F, #0				' clear all flags
		shr	alu, #1		WC		' alu in bits 6..0, bit #0 in carry
		muxc	F, #CF				' set C from bit #0
		muxnz	alu, #$80			' set bit #7 from former carry
		call	#szp_flags_ff
alu_rr_ret
		ret

'******************************************************************************
'
' Compute shift left arithmetic and the flags
'
'    alu: [7][6][5][4][3][2][1][0]  carry: [0]
' before   h  g  f  e  d  c  b  a           ?
' after    g  f  e  d  c  b  a  0           h
'
alu_sla
		mov	F, #0				' clear all flags
		shl	alu, #1				' alu in bits 8..1
		test	alu, #$100	WC		' test former bit #7
		muxc	F, #CF				' set C flag from bit #7
		call	#szp_flags_ff
alu_sla_ret
		ret

'******************************************************************************
'
' Compute shift right arithmetic and the flags
'
'    alu: [7][6][5][4][3][2][1][0]  carry: [0]
' before   h  g  f  e  d  c  b  a           ?
' after    h  h  g  f  e  d  c  b           a
'
alu_sra
		mov	F, #0				' clear all flags
		test	alu, #$01	WC
		muxc	F, #CF				' set C flag on carry
		shl	alu, #24			' alu in bits 31..24
		sar	alu, #25			' alu in bits 6..0, bit #0 in carry
		call	#szp_flags_ff
alu_sra_ret
		ret

'******************************************************************************
'
' Compute shift left inverted and the flags
'
'    alu: [7][6][5][4][3][2][1][0]  carry: [0]
' before   h  g  f  e  d  c  b  a           ?
' after    g  f  e  d  c  b  a  1           h
'
alu_sli
		mov	F, #0				' clear all flags
		shl	alu, #1				' alu in bits 31..25
		test	alu, #$100	WC		' test former bit #7
		or	alu, #$01			' set bit #0
		muxc	F, #CF				' set C flag on carry
		call	#szp_flags_ff
alu_sli_ret
		ret

'******************************************************************************
'
' Compute shift right logical and the flags
'
'    alu: [7][6][5][4][3][2][1][0]  carry: [0]
' before   h  g  f  e  d  c  b  a           ?
' after    0  h  g  f  e  d  c  b           a
'
alu_srl
		mov	F, #0				' clear all flags
		shr	alu, #1		WC		' alu in bits 6..0
		muxc	F, #CF				' set C flag on carry
		call	#szp_flags_ff
alu_srl_ret
		ret

'******************************************************************************
'
' Compute bit test flags
'
alu_bit
		and	alu, tmp	WZ		' mask bit
		muxz	F, #ZF				' set Z flag from ~bit
		test	alu, #$80	WC		' test sign bit
		muxc	F, #SF				' set S flag depending on carry
		andn	F, #NF				' clear N flag
		or	F, #HF				' set H flag
		jmp	#fetch

'******************************************************************************
'
' Pop a word off the Z80 stack
'
pop_alu
		mov	ea, SP
		call	#rd_word
		add	SP, #2
		and	SP, low_word
pop_alu_ret
		ret

'******************************************************************************
'
' Push a word to the Z80 stack
'
push_alu
		sub	SP, #2
		and	SP, low_word
		mov	ea, SP
		call	#wr_word
push_alu_ret
		ret

'******************************************************************************
'
' Return from subroutine
'
do_ret
		call	#pop_alu
		mov	PC, alu
		jmp	#fetch

'******************************************************************************
'
' Call subroutine
'
do_call
		mov	tmp, alu
		mov	alu, PC
		call	#push_alu
		mov	PC, tmp
		jmp	#fetch

'******************************************************************************
'
' Build memory address IX/IY + disp8 in ea
'
mem_xy
		call	#rd_opcode
		rdword	ea, XY
		shl	alu, #24
		sar	alu, #24
		add	ea, alu
mem_xy_ret
		ret

'******************************************************************************
'
' Increment the BC register pair
'
k_inc_bc
		add	C, #1
		test	C, #$100	WC
		addx	B, #0
		and	C, #$ff
		and	B, #$ff
k_inc_bc_ret
		ret

'******************************************************************************
'
' Decrement the BC register pair
'
' returns zero flag if B and C are zero
'
k_dec_bc
		sub	C, #1		WC
		subx	B, #0
		and	C, #$ff		WZ
	if_nz	and	B, #$ff
	if_z	and	B, #$ff		WZ
k_dec_bc_ret
		ret


'******************************************************************************
'
' Increment the DE register pair
'
k_inc_de
		add	E, #1
		test	E, #$100	WC
		addx	D, #0
		and	E, #$ff
		and	D, #$ff
k_inc_de_ret
		ret

'******************************************************************************
'
' Decrement the DE register pair
'
k_dec_de
		sub	E, #1		WC
		subx	D, #0
		and	E, #$ff
		and	D, #$ff
k_dec_de_ret
		ret

'******************************************************************************
'
' Increment the HL register pair
'
k_inc_hl
		add	L, #1
		test	L, #$100	WC
		addx	H, #0
		and	L, #$ff
		and	H, #$ff
k_inc_hl_ret
		ret

'******************************************************************************
'
' Decrement the HL register pair
'
k_dec_hl
		sub	L, #1		WC
		subx	H, #0
		and	L, #$ff
		and	H, #$ff
k_dec_hl_ret
		ret

'******************************************************************************
'
' Store all registers in their hub RAM locations and send a break command
'
k_break
		mov	t1, #C				' source in cog RAM
		mov	ea, regram_ptr			' destination in hub RAM
		mov	aux, #8				' 8 registers
:loop1		movs	:padr1, t1
		add	t1, #1
:padr1		mov	alu, 0-0			' fetch register into alu
		wrbyte	alu, ea				' store in hub RAM
		add	ea, #1
		djnz	aux, #:loop1
		add	ea, #8
		wrbyte	R, ea
		add	ea, #1
		wrbyte	R2, ea
		add	ea, #1
		wrbyte	IFF, ea
		add	ea, #(@@@SP_reg - @@@IFF_reg)
		wrword	SP, ea
		add	ea, #2
		wrword	PC, ea
		wrword	regram_ptr, io_port		' set io_port to the hub RAM address of the registers
		mov	t1, #io_cmd_break		' send a break command to the io handler
		wrbyte	t1, io_command
:wait		rdbyte	t1, io_command		WZ
	if_nz	jmp	#:wait
k_break_ret
		ret

		fit	$1f0

' Don't change the order of the registers unless you also
' change the order of their cog RAM counterparts and the
' k_break function
C_reg		byte	0
B_reg		byte	0
E_reg		byte	0
D_reg		byte	0
L_reg		byte	0
H_reg		byte	0
F_reg		byte	0
A_reg		byte	0
C2_reg		byte	0
B2_reg		byte	0
E2_reg		byte	0
D2_reg		byte	0
L2_reg		byte	0
H2_reg		byte	0
F2_reg		byte	0
A2_reg		byte	0
R_reg		byte	0
R2_reg		byte	0
IFF_reg		byte	0
I_reg		byte	0
SP_reg		word	0
PC_reg		word	0
IX_reg		word	0
IY_reg		word	0

'******************************************************************************
'
'		OPCODES 00
'
'******************************************************************************

' -------------------------------------------------------
z_nop		' 00
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_ex_af_af2	' 08
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	ea, regram_ptr
		add	ea, #8+6			' offset to F2, A2
		rdword	tmp, ea				' fetch F2 and A2
		shl	A, #8				' A = AF
		or	A, F
		xor	A, tmp				' swap AF and AF2
		xor	tmp, A
		xor	A, tmp
		wrword	tmp, ea				' write F and A
		mov	F, A				' restore F
		and	F, #$ff
		shr	A, #8				' restore A
		jmp	#fetch
' -------------------------------------------------------
z_djnz		' 10
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		sub	alu, #1		WZ
		mov	B, alu
		call	#rd_opcode
	if_nz	shl	alu, #24
	if_nz	sar	alu, #24
	if_nz	add	PC, alu
#ifdef COUNT_C
	if_nu	sub	cc, #5
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_jr		' 18
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_opcode
		shl	alu, #24
		sar	alu, #24
		add	PC, alu
		jmp	#fetch
' -------------------------------------------------------
z_jr_nz		' 20
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		test	F, #ZF	WZ
		call	#rd_opcode
	if_z	shl	alu, #24
	if_z	sar	alu, #24
	if_z	add	PC, alu
#ifdef COUNT_C
	if_z	sub	cc, #5
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_jr_z		' 28
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		test	F, #ZF	WZ
		call	#rd_opcode
	if_nz	shl	alu, #24
	if_nz	sar	alu, #24
	if_nz	add	PC, alu
#ifdef COUNT_C
	if_nz	sub	cc, #5
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_jr_nc		' 30
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		test	F, #CF	WZ
		call	#rd_opcode
	if_z	shl	alu, #24
	if_z	sar	alu, #24
	if_z	add	PC, alu
#ifdef COUNT_C
	if_z	sub	cc, #5
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_jr_c		' 38
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		test	F, #CF	WZ
		call	#rd_opcode
	if_nz	shl	alu, #24
	if_nz	sar	alu, #24
	if_nz	add	PC, alu
#ifdef COUNT_C
	if_nz	sub	cc, #5
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_bc_imm16	' 01
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opcode
		mov	C, alu
		call	#rd_opcode
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_de_imm16	' 11
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opcode
		mov	E, alu
		call	#rd_opcode
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_hl_imm16	' 21
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opcode
		mov	L, alu
		call	#rd_opcode
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_sp_imm16	' 31
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		mov	SP, alu
		jmp	#fetch
' -------------------------------------------------------
z_add_hl_bc	' 09
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	tmp, B
		shl	tmp, #8
		or	tmp, C
		jmp	#alu_add16_hl
' -------------------------------------------------------
z_add_hl_de	' 19
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	tmp, D
		shl	tmp, #8
		or	tmp, E
		jmp	#alu_add16_hl
' -------------------------------------------------------
z_add_hl_hl	' 29
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	tmp, H
		shl	tmp, #8
		or	tmp, L
		jmp	#alu_add16_hl
' -------------------------------------------------------
z_add_hl_sp	' 39
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	tmp, SP
		jmp	#alu_add16_hl
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_mem_bc_a	' 02
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	ea, B
		shl	ea, #8
		or	ea, C
		mov	alu, A
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_mem_bc	' 0a
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	ea, B
		shl	ea, #8
		or	ea, C
		call	#rd_byte
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_mem_de_a	' 12
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	ea, D
		shl	ea, #8
		or	ea, E
		mov	alu, A
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_mem_de	' 1a
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	ea, D
		shl	ea, #8
		or	ea, E
		call	#rd_byte
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_abs16_hl	' 22
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		mov	alu, H
		shl	alu, #8
		or	alu, L
		call	#wr_word
		jmp	#fetch
' -------------------------------------------------------
z_ld_hl_abs16	' 2a
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		call	#rd_word
		mov	L, alu
		mov	H, alu
		and	L, #$ff
		shr	H, #8
		jmp	#fetch
' -------------------------------------------------------
z_ld_abs16_a	' 32
#ifdef COUNT_C
		sub	cc, #13
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		mov	alu, A
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_abs16	' 3a
#ifdef COUNT_C
		sub	cc, #13
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		call	#rd_byte
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_inc_bc	' 03
#ifdef COUNT_C
		sub	cc, #6
#endif COUNT_C
		call	#k_inc_bc
		jmp	#fetch
' -------------------------------------------------------
z_dec_bc	' 0b
#ifdef COUNT_C
		sub	cc, #6
#endif COUNT_C
		call	#k_dec_bc
		jmp	#fetch
' -------------------------------------------------------
z_inc_de	' 13
#ifdef COUNT_C
		sub	cc, #6
#endif COUNT_C
		call	#k_inc_de
		jmp	#fetch
' -------------------------------------------------------
z_dec_de	' 1b
#ifdef COUNT_C
		sub	cc, #6
#endif COUNT_C
		call	#k_dec_de
		jmp	#fetch
' -------------------------------------------------------
z_inc_hl	' 23
#ifdef COUNT_C
		sub	cc, #6
#endif COUNT_C
		call	#k_inc_hl
		jmp	#fetch
' -------------------------------------------------------
z_dec_hl	' 2b
#ifdef COUNT_C
		sub	cc, #6
#endif COUNT_C
		call	#k_dec_hl
		jmp	#fetch
' -------------------------------------------------------
z_inc_sp	' 33
#ifdef COUNT_C
		sub	cc, #6
#endif COUNT_C
		add	SP, #1
		and	SP, low_word
		jmp	#fetch
' -------------------------------------------------------
z_dec_sp	' 3b
#ifdef COUNT_C
		sub	cc, #6
#endif COUNT_C
		sub	SP, #1
		and	SP, low_word
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_inc_b		' 04
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, B
		call	#alu_inc
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_inc_c		' 0c
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, C
		call	#alu_inc
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_inc_d		' 14
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, D
		call	#alu_inc
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_inc_e		' 1c
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, E
		call	#alu_inc
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_inc_h		' 24
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, H
		call	#alu_inc
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_inc_l		' 2c
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, L
		call	#alu_inc
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_inc_m		' 34
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		call	#rd_byte_hl
		call	#alu_inc
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_inc_a		' 3c
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, A
		call	#alu_inc
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_dec_b		' 05
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, B
		call	#alu_dec
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_dec_c		' 0d
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, C
		call	#alu_dec
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_dec_d		' 15
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, D
		call	#alu_dec
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_dec_e		' 1d
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, E
		call	#alu_dec
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_dec_h		' 25
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, H
		call	#alu_dec
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_dec_l		' 2d
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, L
		call	#alu_dec
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_dec_m		' 35
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		call	#rd_byte_hl
		call	#alu_dec
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_dec_a		' 3d
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, A
		call	#alu_dec
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_b_imm8	' 06
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_c_imm8	' 0e
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_d_imm8	' 16
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_e_imm8	' 1e
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_h_imm8	' 26
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_l_imm8	' 2e
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_m_imm8	' 36
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opcode
		mov	ea, H
		shl	ea, #8
		or	ea, L
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_imm8	' 3e
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_rlca		' 07
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		shl	A, #1
		test	A, #$100	WC		' check carry
		and	A, #$ff				' keep bits 7..0
		muxc	A, #$01				' set low bit from carry
		and	F, #(SF | ZF | PF)		' keep S, Z and P flags
#ifdef XYFLAGS
		mov	aux, A
		and	aux, #(YF | XF | CF)
		or	F, aux				' set Y, X and C flags
#else
		muxc	F, #CF				' set C flag from carry
#endif XYFLAGS
		jmp	#fetch
' -------------------------------------------------------
z_rrca		' 0f
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		shr	A, #1		WC		' rotate right and check carry
		muxc	A, #$80				' set bit #7 from carry flag
		and	F, #(SF | ZF | PF)		' keep S, Z and P flags
		muxc	F, #CF				' set C from carry flag
#ifdef XYFLAGS
		mov	aux, A
		and	aux, #(YF | XF)
		or	F, aux				' set Y and X flags
#endif XYFLAGS
		jmp	#fetch
' -------------------------------------------------------
z_rla		' 17
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		shl	A, #1
		test	A, #$100	WZ		' test bit #7
		and	A, #$ff				' keep bits 7..0
		test	F, #CF	WC			' get C flag to carry
		muxnz	F, #CF				' set C from zero flag
		muxc	A, #$01				' set bit #0 from previous C flag
		and	F, #(SF | ZF | PF | CF)		' keep S, Z, P and C flags
#ifdef XYFLAGS
		mov	aux, A
		and	aux, #(YF | XF)
		or	F, aux				' set Y and X flags
#endif XYFLAGS
		jmp	#fetch
' -------------------------------------------------------
z_rra		' 1f
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		shr	A, #1		WC		' rotate right and check carry
		test	F, #CF	WZ			' get C flag
		muxc	F, #CF				' set C flag from carry flag
		muxnz	A, #$80				' set bit #7 from zero flag
		and	F, #(SF | ZF | PF | CF)		' keep S, Z, P and C flags
#ifdef XYFLAGS
		mov	aux, A
		and	aux, #(YF | XF)
		or	F, aux				' set Y and X flags
#endif XYFLAGS
		jmp	#fetch
' -------------------------------------------------------
z_daa		' 27
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	alu, A
		mov	tmp, alu
		mov	aux, alu			' save for auxiliary flag test
		and	tmp, #15			' isolate low nibble of accumulator
		mov	daa_adj, #$06			' +$06
		test	F, #NF		WZ		' test negative flag
		negnz	daa_adj, daa_adj		' -$06
		test	F, #HF		WZ
		cmp	tmp, #$09+1	WC		' aux isn't set; is nibble > $9 ?
  if_nc_or_nz	add	A, daa_adj			' aux was set or nibble is > $9 => add $06 or -$06
		shl	daa_adj, #4			' +$60 or -$60
		test	F, #CF		WZ
		cmp	alu, #$99+1	WC		' carry isn't set; is data > $99 ?
  if_nc_or_nz	add	A, daa_adj			' carry was set or data is > $99 => add $60 or -$60
		and	F, #(NF | CF)			' preserve old CF and NF
		cmp	alu, #$99+1	WC
	if_nc	or	F, #CF				' set carry if result > $99
		xor	aux, A
		and	aux, #HF	WZ		' if bit #4 of result changed
		or	F, aux				' set the auxiliary carry flag
		and	A, #$ff		WZ, WC		' get zero and parity flags
		muxz	F, #ZF				' set Z on zero
		muxnc	F, #PF				' clear P on parity
		test	A, #$80		WZ		' get sign bit #7
		muxnz	F, #SF				' set S on sign
		jmp	#fetch
' -------------------------------------------------------
z_cpl		' 2f
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		xor	A, #$ff
		or	F, #(HF | NF)			' set the H and N flags
#ifdef XYFLAGS
		andn	F, #(XF | YF)			' clear the X and Y flags
		mov	aux, A
		and	aux, #(XF | YF)
		or	F, aux				' set the X and Y flags
#endif XYFLAGS
		jmp	#fetch
' -------------------------------------------------------
z_scf		' 37
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		or	F, #CF				' set the C flag
		andn	F, #(HF | NF)			' clear the H and N flags
#ifdef XYFLAGS
		mov	aux, A
		and	aux, #(XF | YF)
		or	F, aux				' or in the X and Y flags (sic!)
#endif XYFLAGS
		jmp	#fetch
' -------------------------------------------------------
z_ccf		' 3f
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		test	F, #CF		WC		' get the C flag into the carry
		muxc	F, #HF				' set the H flag if C was set
		xor	F, #CF				' toggle the C flag
		andn	F, #NF				' clear N flag
#ifdef XYFLAGS
		mov	aux, A
		and	aux, #(XF | YF)
		or	F, aux				' or in the X and Y flags (sic!)
#endif XYFLAGS
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_b_b	' 40
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_ld_b_c	' 41
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	B, C
		jmp	#fetch
' -------------------------------------------------------
z_ld_b_d	' 42
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	B, D
		jmp	#fetch
' -------------------------------------------------------
z_ld_b_e	' 43
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	B, E
		jmp	#fetch
' -------------------------------------------------------
z_ld_b_h	' 44
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	B, H
		jmp	#fetch
' -------------------------------------------------------
z_ld_b_l	' 45
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	B, L
		jmp	#fetch
' -------------------------------------------------------
z_ld_b_m	' 46
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_b_a	' 47
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	B, A
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_c_b	' 48
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	C, B
		jmp	#fetch
' -------------------------------------------------------
z_ld_c_c	' 49
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_ld_c_d	' 4a
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	C, D
		jmp	#fetch
' -------------------------------------------------------
z_ld_c_e	' 4b
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	C, E
		jmp	#fetch
' -------------------------------------------------------
z_ld_c_h	' 4c
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	C, H
		jmp	#fetch
' -------------------------------------------------------
z_ld_c_l	' 4d
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	C, L
		jmp	#fetch
' -------------------------------------------------------
z_ld_c_m	' 4e
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_c_a	' 4f
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	C, A
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_d_b	' 50
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	D, B
		jmp	#fetch
' -------------------------------------------------------
z_ld_d_c	' 51
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	D, C
		jmp	#fetch
' -------------------------------------------------------
z_ld_d_d	' 52
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_ld_d_e	' 53
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	D, E
		jmp	#fetch
' -------------------------------------------------------
z_ld_d_h	' 54
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	D, H
		jmp	#fetch
' -------------------------------------------------------
z_ld_d_l	' 55
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	D, L
		jmp	#fetch
' -------------------------------------------------------
z_ld_d_m	' 56
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_d_a	' 57
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	D, A
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_e_b	' 58
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	E, B
		jmp	#fetch
' -------------------------------------------------------
z_ld_e_c	' 59
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	E, C
		jmp	#fetch
' -------------------------------------------------------
z_ld_e_d	' 5a
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	E, D
		jmp	#fetch
' -------------------------------------------------------
z_ld_e_e	' 5b
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_ld_e_h	' 5c
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	E, H
		jmp	#fetch
' -------------------------------------------------------
z_ld_e_l	' 5d
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	E, L
		jmp	#fetch
' -------------------------------------------------------
z_ld_e_m	' 5e
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_e_a	' 5f
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	E, A
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_h_b	' 60
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	H, B
		jmp	#fetch
' -------------------------------------------------------
z_ld_h_c	' 61
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	H, C
		jmp	#fetch
' -------------------------------------------------------
z_ld_h_d	' 62
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	H, D
		jmp	#fetch
' -------------------------------------------------------
z_ld_h_e	' 63
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	H, E
		jmp	#fetch
' -------------------------------------------------------
z_ld_h_h	' 64
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_ld_h_l	' 65
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	H, L
		jmp	#fetch
' -------------------------------------------------------
z_ld_h_m	' 66
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_h_a	' 67
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	H, A
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_l_b	' 68
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	L, B
		jmp	#fetch
' -------------------------------------------------------
z_ld_l_c	' 69
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	L, C
		jmp	#fetch
' -------------------------------------------------------
z_ld_l_d	' 6a
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	L, D
		jmp	#fetch
' -------------------------------------------------------
z_ld_l_e	' 6b
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	L, E
		jmp	#fetch
' -------------------------------------------------------
z_ld_l_h	' 6c
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	L, H
		jmp	#fetch
' -------------------------------------------------------
z_ld_l_l	' 6d
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_ld_l_m	' 6e
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_l_a	' 6f
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	L, A
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_m_b	' 70
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	alu, B
		mov	ea, H
		shl	ea, #8
		or	ea, L
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_m_c	' 71
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	alu, C
		mov	ea, H
		shl	ea, #8
		or	ea, L
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_m_d	' 72
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	alu, D
		mov	ea, H
		shl	ea, #8
		or	ea, L
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_m_e	' 73
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	alu, E
		mov	ea, H
		shl	ea, #8
		or	ea, L
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_m_h	' 74
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	alu, H
		mov	ea, H
		shl	ea, #8
		or	ea, L
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_m_l	' 75
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	alu, L
		mov	ea, H
		shl	ea, #8
		or	ea, L
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_halt		' 76
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		' FIXME: halt
		call	#k_break
		jmp	#fetch
' -------------------------------------------------------
z_ld_m_a	' 77
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	alu, A
		mov	ea, H
		shl	ea, #8
		or	ea, L
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_a_b	' 78
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	A, B
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_c	' 79
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		mov	A, C
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_d	' 7a
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	A, D
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_e	' 7b
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	A, E
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_h	' 7c
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	A, H
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_l	' 7d
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	A, L
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_m	' 7e
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_a	' 7f
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_add_a_b	' 80
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, B
		jmp	#alu_add
' -------------------------------------------------------
z_add_a_c	' 81
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, C
		jmp	#alu_add
' -------------------------------------------------------
z_add_a_d	' 82
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, D
		jmp	#alu_add
' -------------------------------------------------------
z_add_a_e	' 83
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, E
		jmp	#alu_add
' -------------------------------------------------------
z_add_a_h	' 84
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, H
		jmp	#alu_add
' -------------------------------------------------------
z_add_a_l	' 85
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, L
		jmp	#alu_add
' -------------------------------------------------------
z_add_a_m	' 86
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		jmp	#alu_add
' -------------------------------------------------------
z_add_a_a	' 87
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, A
		jmp	#alu_add

' -------------------------------------------------------
z_adc_a_b	' 88
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, B
		jmp	#alu_adc
' -------------------------------------------------------
z_adc_a_c	' 89
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, C
		jmp	#alu_adc
' -------------------------------------------------------
z_adc_a_d	' 8a
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, D
		jmp	#alu_adc
' -------------------------------------------------------
z_adc_a_e	' 8b
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, E
		jmp	#alu_adc
' -------------------------------------------------------
z_adc_a_h	' 8c
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, H
		jmp	#alu_adc
' -------------------------------------------------------
z_adc_a_l	' 8d
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, L
		jmp	#alu_adc
' -------------------------------------------------------
z_adc_a_m	' 8e
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		jmp	#alu_adc
' -------------------------------------------------------
z_adc_a_a	' 8f
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, A
		jmp	#alu_adc
' -------------------------------------------------------

' -------------------------------------------------------
z_sub_b		' 90
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, B
		jmp	#alu_sub
' -------------------------------------------------------
z_sub_c		' 91
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, C
		jmp	#alu_sub
' -------------------------------------------------------
z_sub_d		' 92
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, D
		jmp	#alu_sub
' -------------------------------------------------------
z_sub_e		' 93
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, E
		jmp	#alu_sub
' -------------------------------------------------------
z_sub_h		' 94
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, H
		jmp	#alu_sub
' -------------------------------------------------------
z_sub_l		' 95
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, L
		jmp	#alu_sub
' -------------------------------------------------------
z_sub_m		' 96
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		jmp	#alu_sub
' -------------------------------------------------------
z_sub_a		' 97
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, A
		jmp	#alu_sub
' -------------------------------------------------------

' -------------------------------------------------------
z_sbc_a_b	' 98
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, B
		jmp	#alu_sbc
' -------------------------------------------------------
z_sbc_a_c	' 99
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, C
		jmp	#alu_sbc
' -------------------------------------------------------
z_sbc_a_d	' 9a
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, D
		jmp	#alu_sbc
' -------------------------------------------------------
z_sbc_a_e	' 9b
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, E
		jmp	#alu_sbc
' -------------------------------------------------------
z_sbc_a_h	' 9c
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, H
		jmp	#alu_sbc
' -------------------------------------------------------
z_sbc_a_l	' 9d
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, L
		jmp	#alu_sbc
' -------------------------------------------------------
z_sbc_a_m	' 9e
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		jmp	#alu_sbc
' -------------------------------------------------------
z_sbc_a_a	' 9f
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, A
		jmp	#alu_sbc
' -------------------------------------------------------

' -------------------------------------------------------
z_and_b		' a0
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, B
		jmp	#alu_and
' -------------------------------------------------------
z_and_c		' a1
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, C
		jmp	#alu_and
' -------------------------------------------------------
z_and_d		' a2
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, D
		jmp	#alu_and
' -------------------------------------------------------
z_and_e		' a3
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, E
		jmp	#alu_and
' -------------------------------------------------------
z_and_h		' a4
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, H
		jmp	#alu_and
' -------------------------------------------------------
z_and_l		' a5
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, L
		jmp	#alu_and
' -------------------------------------------------------
z_and_m		' a6
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		jmp	#alu_and
' -------------------------------------------------------
z_and_a		' a7
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, A
		jmp	#alu_and
' -------------------------------------------------------

' -------------------------------------------------------
z_xor_b		' a8
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, B
		jmp	#alu_xor
' -------------------------------------------------------
z_xor_c		' a9
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, C
		jmp	#alu_xor
' -------------------------------------------------------
z_xor_d		' aa
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, D
		jmp	#alu_xor
' -------------------------------------------------------
z_xor_e		' ab
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, E
		jmp	#alu_xor
' -------------------------------------------------------
z_xor_h		' ac
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, H
		jmp	#alu_xor
' -------------------------------------------------------
z_xor_l		' ad
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, L
		jmp	#alu_xor
' -------------------------------------------------------
z_xor_m		' ae
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		jmp	#alu_xor
' -------------------------------------------------------
z_xor_a		' af
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, A
		jmp	#alu_xor
' -------------------------------------------------------

' -------------------------------------------------------
z_or_b		' b0
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, B
		jmp	#alu_or
' -------------------------------------------------------
z_or_c		' b1
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, C
		jmp	#alu_or
' -------------------------------------------------------
z_or_d		' b2
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, D
		jmp	#alu_or
' -------------------------------------------------------
z_or_e		' b3
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, E
		jmp	#alu_or
' -------------------------------------------------------
z_or_h		' b4
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, H
		jmp	#alu_or
' -------------------------------------------------------
z_or_l		' b5
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, L
		jmp	#alu_or
' -------------------------------------------------------
z_or_m		' b6
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		jmp	#alu_or
' -------------------------------------------------------
z_or_a		' b7
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, A
		jmp	#alu_or
' -------------------------------------------------------

' -------------------------------------------------------
z_cp_b		' b8
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, B
		jmp	#alu_cp
' -------------------------------------------------------
z_cp_c		' b9
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, C
		jmp	#alu_cp
' -------------------------------------------------------
z_cp_d		' ba
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, D
		jmp	#alu_cp
' -------------------------------------------------------
z_cp_e		' bb
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, E
		jmp	#alu_cp
' -------------------------------------------------------
z_cp_h		' bc
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, H
		jmp	#alu_cp
' -------------------------------------------------------
z_cp_l		' bd
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, L
		jmp	#alu_cp
' -------------------------------------------------------
z_cp_m		' be
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		jmp	#alu_cp
' -------------------------------------------------------
z_cp_a		' bf
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	tmp, A
		jmp	#alu_cp
' -------------------------------------------------------

' -------------------------------------------------------
z_ret_nz	' c0
#ifdef COUNT_C
		sub	cc, #5
#endif COUNT_C
		test	F, #ZF	WZ
#ifdef COUNT_C
	if_z	sub	cc, #6
#endif COUNT_C
	if_z	jmp	#do_ret
		jmp	#fetch
' -------------------------------------------------------
z_ret_z		' c8
#ifdef COUNT_C
		sub	cc, #5
#endif COUNT_C
		test	F, #ZF	WZ
#ifdef COUNT_C
	if_nz	sub	cc, #6
#endif COUNT_C
	if_nz	jmp	#do_ret
		jmp	#fetch
' -------------------------------------------------------
z_ret_nc	' d0
#ifdef COUNT_C
		sub	cc, #5
#endif COUNT_C
		test	F, #CF	WZ
#ifdef COUNT_C
	if_z	sub	cc, #6
#endif COUNT_C
	if_z	jmp	#do_ret
		jmp	#fetch
' -------------------------------------------------------
z_ret_c		' d8
#ifdef COUNT_C
		sub	cc, #5
#endif COUNT_C
		test	F, #CF	WZ
#ifdef COUNT_C
	if_nz	sub	cc, #6
#endif COUNT_C
	if_nz	jmp	#do_ret
		jmp	#fetch
' -------------------------------------------------------
z_ret_po	' e0
#ifdef COUNT_C
		sub	cc, #5
#endif COUNT_C
		test	F, #PF	WZ
#ifdef COUNT_C
	if_z	sub	cc, #6
#endif COUNT_C
	if_z	jmp	#do_ret
		jmp	#fetch
' -------------------------------------------------------
z_ret_pe	' e8
#ifdef COUNT_C
		sub	cc, #5
#endif COUNT_C
		test	F, #PF	WZ
#ifdef COUNT_C
	if_nz	sub	cc, #6
#endif COUNT_C
	if_nz	jmp	#do_ret
		jmp	#fetch
' -------------------------------------------------------
z_ret_p		' f0
#ifdef COUNT_C
		sub	cc, #5
#endif COUNT_C
		test	F, #SF	WZ
#ifdef COUNT_C
	if_z	sub	cc, #6
#endif COUNT_C
	if_z	jmp	#do_ret
		jmp	#fetch
' -------------------------------------------------------
z_ret_m		' f8
#ifdef COUNT_C
		sub	cc, #5
#endif COUNT_C
		test	F, #SF	WZ
#ifdef COUNT_C
	if_nz	sub	cc, #6
#endif COUNT_C
	if_nz	jmp	#do_ret
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_pop_bc	' c1
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#pop_alu
		mov	C, alu
		mov	B, alu
		and	C, #$ff
		shr	B, #8
		jmp	#fetch
' -------------------------------------------------------
z_pop_de	' d1
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#pop_alu
		mov	E, alu
		mov	D, alu
		and	E, #$ff
		shr	D, #8
		jmp	#fetch
' -------------------------------------------------------
z_pop_hl	' e1
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#pop_alu
		mov	L, alu
		mov	H, alu
		and	L, #$ff
		shr	H, #8
		jmp	#fetch
' -------------------------------------------------------
z_pop_af	' f1
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#pop_alu
		mov	F, alu
		mov	A, alu
		and	F, #$ff
		shr	A, #8
		jmp	#fetch
' -------------------------------------------------------
z_ret		' c9
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		jmp	#do_ret
' -------------------------------------------------------
z_exx		' d9
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	ea, regram_ptr
		add	ea, #8			' offset to C2
		rdword	tmp, ea
		shl	B, #8
		or	B, C
		xor	B, tmp
		xor	tmp, B
		xor	B, tmp
		wrword	tmp, ea
		mov	C, B
		and	C, #$ff
		shr	B, #8
		add	ea, #2

		rdword	tmp, ea
		shl	D, #8
		or	D, E
		xor	D, tmp
		xor	tmp, D
		xor	D, tmp
		wrword	tmp, ea
		mov	E, D
		and	E, #$ff
		shr	D, #8
		add	ea, #2

		rdword	tmp, ea
		shl	H, #8
		or	H, L
		xor	H, tmp
		xor	tmp, H
		xor	H, tmp
		wrword	tmp, ea
		mov	L, H
		and	L, #$ff
		shr	H, #8

		jmp	#fetch
' -------------------------------------------------------
z_jp_hl		' e9
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		mov	PC, H
		shl	PC, #8
		or	PC, L
		jmp	#fetch
' -------------------------------------------------------
z_ld_sp_hl	' f9
#ifdef COUNT_C
		sub	cc, #6
#endif COUNT_C
		mov	SP, H
		shl	SP, #8
		or	SP, L
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_jp_nz		' c2
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #ZF	WZ
	if_z	mov	PC, alu
		jmp	#fetch
' -------------------------------------------------------
z_jp_z		' ca
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #ZF	WZ
	if_nz	mov	PC, alu
		jmp	#fetch
' -------------------------------------------------------
z_jp_nc		' d2
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #CF	WZ
	if_z	mov	PC, alu
		jmp	#fetch
' -------------------------------------------------------
z_jp_c		' da
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #CF	WZ
	if_nz	mov	PC, alu
		jmp	#fetch
' -------------------------------------------------------
z_jp_po		' e2
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #PF	WZ
	if_z	mov	PC, alu
		jmp	#fetch
' -------------------------------------------------------
z_jp_pe		' ea
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #PF	WZ
	if_nz	mov	PC, alu
		jmp	#fetch
' -------------------------------------------------------
z_jp_p		' f2
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #SF	WZ
	if_z	mov	PC, alu
		jmp	#fetch
' -------------------------------------------------------
z_jp_m		' fa
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #SF	WZ
	if_nz	mov	PC, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_jp		' c3
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		mov	PC, alu
		jmp	#fetch
' -------------------------------------------------------
z_cb_xx		' cb
		mov	tmp, table_cb
		jmp	#fetch1
' -------------------------------------------------------
z_out		' d3
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		call	#rd_opcode
		mov	ea, A
		shl	ea, #8
		or	ea, alu
		mov	alu, A
		call	#wr_port
		jmp	#fetch
' -------------------------------------------------------
z_in		' db
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		call	#rd_opcode
		mov	ea, A
		shl	ea, #8
		or	ea, alu
		call	#rd_port
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------
z_ex_sp_hl	' e3
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		mov	ea, SP
		call	#rd_byte
		xor	L, alu
		xor	alu, L
		xor	L, alu
		call	#wr_byte
		add	ea, #1
		call	#rd_byte
		xor	H, alu
		xor	alu, H
		xor	H, alu
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ex_de_hl	' eb
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		xor	L, E
		xor	E, L
		xor	L, E
		xor	H, D
		xor	D, H
		xor	H, D
		jmp	#fetch
' -------------------------------------------------------
z_di		' f3
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		andn	IFF, #(IFF1|IFF2)		' disable IFF1 and IFF2
		jmp	#fetch
' -------------------------------------------------------
z_ei		' fb
#ifdef COUNT_C
		sub	cc, #4
#endif COUNT_C
		or	IFF, #(IFF1|IFF2)		' enable IFF1 and IFF2
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_call_nz	' c4
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #ZF	WZ
#ifdef COUNT_C
	if_z	sub	cc, #7
#endif COUNT_C
	if_z	jmp	#do_call
		jmp	#fetch
' -------------------------------------------------------
z_call_z	' cc
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #ZF	WZ
#ifdef COUNT_C
	if_nz	sub	cc, #7
#endif COUNT_C
	if_nz	jmp	#do_call
		jmp	#fetch
' -------------------------------------------------------
z_call_nc	' d4
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #CF	WZ
#ifdef COUNT_C
	if_z	sub	cc, #7
#endif COUNT_C
	if_z	jmp	#do_call
		jmp	#fetch
' -------------------------------------------------------
z_call_c	' dc
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #CF	WZ
#ifdef COUNT_C
	if_nz	sub	cc, #7
#endif COUNT_C
	if_nz	jmp	#do_call
		jmp	#fetch
' -------------------------------------------------------
z_call_po	' e4
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #PF	WZ
#ifdef COUNT_C
	if_z	sub	cc, #7
#endif COUNT_C
	if_z	jmp	#do_call
		jmp	#fetch
' -------------------------------------------------------
z_call_pe	' ec
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #PF	WZ
#ifdef COUNT_C
	if_nz	sub	cc, #7
#endif COUNT_C
	if_nz	jmp	#do_call
		jmp	#fetch
' -------------------------------------------------------
z_call_p	' f4
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #SF	WZ
#ifdef COUNT_C
	if_z	sub	cc, #7
#endif COUNT_C
	if_z	jmp	#do_call
		jmp	#fetch
' -------------------------------------------------------
z_call_m	' fc
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		call	#rd_opword
		test	F, #SF	WZ
#ifdef COUNT_C
	if_nz	sub	cc, #7
#endif COUNT_C
	if_nz	jmp	#do_call
		jmp	#fetch
' -------------------------------------------------------


' -------------------------------------------------------
z_push_bc	' c5
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, B
		shl	alu, #8
		or	alu, C
		call	#push_alu
		jmp	#fetch
' -------------------------------------------------------
z_push_de	' d5
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, D
		shl	alu, #8
		or	alu, E
		call	#push_alu
		jmp	#fetch
' -------------------------------------------------------
z_push_hl	' e5
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, H
		shl	alu, #8
		or	alu, L
		call	#push_alu
		jmp	#fetch
' -------------------------------------------------------
z_push_af	' f5
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, A
		shl	alu, #8
		or	alu, F
		call	#push_alu
		jmp	#fetch
' -------------------------------------------------------
z_call		' cd
#ifdef COUNT_C
		sub	cc, #17
#endif COUNT_C
		call	#rd_opword
		jmp	#do_call
' -------------------------------------------------------
z_dd_xx		' dd
		mov	XY, IX
		mov	tmp, table_xy
		jmp	#fetch1
' -------------------------------------------------------
z_ed_xx		' ed
		mov	tmp, table_ed
#ifdef COUNT_R
		add	R, #1
#endif COUNT_R
		call	#rd_opcode
		sub	alu, #$40	WZ, WC
	if_b	jmp	#fetch				' ignore ED 00 to ED 3F
		cmp	alu, #$c0-$40	WZ, WC
	if_ae	jmp	#fetch				' ignore ED C0 to ED FF
		add	tmp, alu
		add	tmp, alu
		jmp	#fetch2
' -------------------------------------------------------
z_fd_xx		' fd
		mov	XY, IY
		mov	tmp, table_xy
		jmp	#fetch1
' -------------------------------------------------------

' -------------------------------------------------------
z_add_a_imm8	' c6
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	tmp, alu
		jmp	#alu_add
' -------------------------------------------------------
z_adc_a_imm8	' ce
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	tmp, alu
		jmp	#alu_adc
' -------------------------------------------------------
z_sub_imm8	' d6
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	tmp, alu
		jmp	#alu_sub
' -------------------------------------------------------
z_sbc_a_imm8	' de
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	tmp, alu
		jmp	#alu_sbc
' -------------------------------------------------------
z_and_imm8	' e6
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	tmp, alu
		jmp	#alu_and
' -------------------------------------------------------
z_xor_imm8	' ee
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	tmp, alu
		jmp	#alu_xor
' -------------------------------------------------------
z_or_imm8	' f6
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	tmp, alu
		jmp	#alu_or
' -------------------------------------------------------
z_cp_imm8	' fe
#ifdef COUNT_C
		sub	cc, #7
#endif COUNT_C
		call	#rd_opcode
		mov	tmp, alu
		jmp	#alu_cp
' -------------------------------------------------------

' -------------------------------------------------------
z_rst_00	' c7
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, #$00
		jmp	#do_call
' -------------------------------------------------------
z_rst_08	' cf
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, #$08
		jmp	#do_call
' -------------------------------------------------------
z_rst_10	' d7
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, #$10
		jmp	#do_call
' -------------------------------------------------------
z_rst_18	' df
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, #$18
		jmp	#do_call
' -------------------------------------------------------
z_rst_20	' e7
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, #$20
		jmp	#do_call
' -------------------------------------------------------
z_rst_28	' ef
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, #$28
		jmp	#do_call
' -------------------------------------------------------
z_rst_30	' f7
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, #$30
		jmp	#do_call
' -------------------------------------------------------
z_rst_38	' ff
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		mov	alu, #$38
		jmp	#do_call
' -------------------------------------------------------


'******************************************************************************
'
'		OPCODES CB
'
'******************************************************************************


' -------------------------------------------------------
z_rlc_b		' cb 00
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		call	#alu_rlc
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_rlc_c		' cb 01
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		call	#alu_rlc
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_rlc_d		' cb 02
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		call	#alu_rlc
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_rlc_e		' cb 03
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		call	#alu_rlc
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_rlc_h		' cb 04
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		call	#alu_rlc
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_rlc_l		' cb 05
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		call	#alu_rlc
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_rlc_m		' cb 06
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		call	#alu_rlc
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_rlc_a		' cb 07
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		call	#alu_rlc
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_rrc_b		' cb 08
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		call	#alu_rrc
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_rrc_c		' cb 09
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		call	#alu_rrc
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_rrc_d		' cb 0a
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		call	#alu_rrc
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_rrc_e		' cb 0b
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		call	#alu_rrc
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_rrc_h		' cb 0c
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		call	#alu_rrc
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_rrc_l		' cb 0d
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		call	#alu_rrc
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_rrc_m		' cb 0e
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		call	#alu_rrc
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_rrc_a		' cb 0f
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		call	#alu_rrc
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_rl_b		' cb 10
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		call	#alu_rl
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_rl_c		' cb 11
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		call	#alu_rl
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_rl_d		' cb 12
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		call	#alu_rl
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_rl_e		' cb 13
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		call	#alu_rl
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_rl_h		' cb 14
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		call	#alu_rl
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_rl_l		' cb 15
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		call	#alu_rl
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_rl_m		' cb 16
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		call	#alu_rl
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_rl_a		' cb 17
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		call	#alu_rl
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_rr_b		' cb 18
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		call	#alu_rr
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_rr_c		' cb 19
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		call	#alu_rr
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_rr_d		' cb 1a
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		call	#alu_rr
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_rr_e		' cb 1b
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		call	#alu_rr
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_rr_h		' cb 1c
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		call	#alu_rr
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_rr_l		' cb 1d
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		call	#alu_rr
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_rr_m		' cb 1e
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		call	#alu_rr
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_rr_a		' cb 1f
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		call	#alu_rr
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_sla_b		' cb 20
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		call	#alu_sla
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_sla_c		' cb 21
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		call	#alu_sla
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_sla_d		' cb 22
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		call	#alu_sla
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_sla_e		' cb 23
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		call	#alu_sla
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_sla_h		' cb 24
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		call	#alu_sla
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_sla_l		' cb 25
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		call	#alu_sla
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_sla_m		' cb 26
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		call	#alu_sla
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_sla_a		' cb 27
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		call	#alu_sla
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_sra_b		' cb 28
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		call	#alu_sra
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_sra_c		' cb 29
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		call	#alu_sra
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_sra_d		' cb 2a
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		call	#alu_sra
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_sra_e		' cb 2b
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		call	#alu_sra
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_sra_h		' cb 2c
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		call	#alu_sra
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_sra_l		' cb 2d
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		call	#alu_sra
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_sra_m		' cb 2e
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		call	#alu_sra
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_sra_a		' cb 2f
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		call	#alu_sra
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_sli_b		' cb 30
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		call	#alu_sli
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_sli_c		' cb 31
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		call	#alu_sli
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_sli_d		' cb 32
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		call	#alu_sli
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_sli_e		' cb 33
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		call	#alu_sli
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_sli_h		' cb 34
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		call	#alu_sli
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_sli_l		' cb 35
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		call	#alu_sli
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_sli_m		' cb 36
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		call	#alu_sli
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_sli_a		' cb 37
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		call	#alu_sli
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_srl_b		' cb 38
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		call	#alu_srl
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_srl_c		' cb 39
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		call	#alu_srl
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_srl_d		' cb 3a
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		call	#alu_srl
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_srl_e		' cb 3b
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		call	#alu_srl
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_srl_h		' cb 3c
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		call	#alu_srl
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_srl_l		' cb 3d
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		call	#alu_srl
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_srl_m		' cb 3e
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		call	#alu_srl
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_srl_a		' cb 3f
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		call	#alu_srl
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_bit0_b	' cb 40
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		mov	tmp, #bit0
		jmp	#alu_bit
' -------------------------------------------------------
z_bit0_c	' cb 41
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		mov	tmp, #bit0
		jmp	#alu_bit
' -------------------------------------------------------
z_bit0_d	' cb 42
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		mov	tmp, #bit0
		jmp	#alu_bit
' -------------------------------------------------------
z_bit0_e	' cb 43
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		mov	tmp, #bit0
		jmp	#alu_bit
' -------------------------------------------------------
z_bit0_h	' cb 44
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		mov	tmp, #bit0
		jmp	#alu_bit
' -------------------------------------------------------
z_bit0_l	' cb 45
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		mov	tmp, #bit0
		jmp	#alu_bit
' -------------------------------------------------------
z_bit0_m	' cb 46
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, #bit0
		jmp	#alu_bit
' -------------------------------------------------------
z_bit0_a	' cb 47
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		mov	tmp, #bit0
		jmp	#alu_bit
' -------------------------------------------------------

' -------------------------------------------------------
z_bit1_b	' cb 48
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		mov	tmp, #bit1
		jmp	#alu_bit
' -------------------------------------------------------
z_bit1_c	' cb 49
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		mov	tmp, #bit1
		jmp	#alu_bit
' -------------------------------------------------------
z_bit1_d	' cb 4a
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		mov	tmp, #bit1
		jmp	#alu_bit
' -------------------------------------------------------
z_bit1_e	' cb 4b
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		mov	tmp, #bit1
		jmp	#alu_bit
' -------------------------------------------------------
z_bit1_h	' cb 4c
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		mov	tmp, #bit1
		jmp	#alu_bit
' -------------------------------------------------------
z_bit1_l	' cb 4d
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		mov	tmp, #bit1
		jmp	#alu_bit
' -------------------------------------------------------
z_bit1_m	' cb 4e
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, #bit1
		jmp	#alu_bit
' -------------------------------------------------------
z_bit1_a	' cb 4f
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		mov	tmp, #bit1
		jmp	#alu_bit
' -------------------------------------------------------

' -------------------------------------------------------
z_bit2_b	' cb 50
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		mov	tmp, #bit2
		jmp	#alu_bit
' -------------------------------------------------------
z_bit2_c	' cb 51
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		mov	tmp, #bit2
		jmp	#alu_bit
' -------------------------------------------------------
z_bit2_d	' cb 52
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		mov	tmp, #bit2
		jmp	#alu_bit
' -------------------------------------------------------
z_bit2_e	' cb 53
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		mov	tmp, #bit2
		jmp	#alu_bit
' -------------------------------------------------------
z_bit2_h	' cb 54
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		mov	tmp, #bit2
		jmp	#alu_bit
' -------------------------------------------------------
z_bit2_l	' cb 55
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		mov	tmp, #bit2
		jmp	#alu_bit
' -------------------------------------------------------
z_bit2_m	' cb 56
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, #bit2
		jmp	#alu_bit
' -------------------------------------------------------
z_bit2_a	' cb 57
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		mov	tmp, #bit2
		jmp	#alu_bit
' -------------------------------------------------------

' -------------------------------------------------------
z_bit3_b	' cb 58
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		mov	tmp, #bit3
		jmp	#alu_bit
' -------------------------------------------------------
z_bit3_c	' cb 59
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		mov	tmp, #bit3
		jmp	#alu_bit
' -------------------------------------------------------
z_bit3_d	' cb 5a
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		mov	tmp, #bit3
		jmp	#alu_bit
' -------------------------------------------------------
z_bit3_e	' cb 5b
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		mov	tmp, #bit3
		jmp	#alu_bit
' -------------------------------------------------------
z_bit3_h	' cb 5c
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		mov	tmp, #bit3
		jmp	#alu_bit
' -------------------------------------------------------
z_bit3_l	' cb 5d
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		mov	tmp, #bit3
		jmp	#alu_bit
' -------------------------------------------------------
z_bit3_m	' cb 5e
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, #bit3
		jmp	#alu_bit
' -------------------------------------------------------
z_bit3_a	' cb 5f
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		mov	tmp, #bit3
		jmp	#alu_bit
' -------------------------------------------------------

' -------------------------------------------------------
z_bit4_b	' cb 60
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		mov	tmp, #bit4
		jmp	#alu_bit
' -------------------------------------------------------
z_bit4_c	' cb 61
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		mov	tmp, #bit4
		jmp	#alu_bit
' -------------------------------------------------------
z_bit4_d	' cb 62
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		mov	tmp, #bit4
		jmp	#alu_bit
' -------------------------------------------------------
z_bit4_e	' cb 63
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		mov	tmp, #bit4
		jmp	#alu_bit
' -------------------------------------------------------
z_bit4_h	' cb 64
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		mov	tmp, #bit4
		jmp	#alu_bit
' -------------------------------------------------------
z_bit4_l	' cb 65
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		mov	tmp, #bit4
		jmp	#alu_bit
' -------------------------------------------------------
z_bit4_m	' cb 66
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, #bit4
		jmp	#alu_bit
' -------------------------------------------------------
z_bit4_a	' cb 67
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		mov	tmp, #bit4
		jmp	#alu_bit
' -------------------------------------------------------

' -------------------------------------------------------
z_bit5_b	' cb 68
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		mov	tmp, #bit5
		jmp	#alu_bit
' -------------------------------------------------------
z_bit5_c	' cb 69
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		mov	tmp, #bit5
		jmp	#alu_bit
' -------------------------------------------------------
z_bit5_d	' cb 6a
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		mov	tmp, #bit5
		jmp	#alu_bit
' -------------------------------------------------------
z_bit5_e	' cb 6b
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		mov	tmp, #bit5
		jmp	#alu_bit
' -------------------------------------------------------
z_bit5_h	' cb 6c
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		mov	tmp, #bit5
		jmp	#alu_bit
' -------------------------------------------------------
z_bit5_l	' cb 6d
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		mov	tmp, #bit5
		jmp	#alu_bit
' -------------------------------------------------------
z_bit5_m	' cb 6e
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, #bit5
		jmp	#alu_bit
' -------------------------------------------------------
z_bit5_a	' cb 6f
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		mov	tmp, #bit5
		jmp	#alu_bit
' -------------------------------------------------------

' -------------------------------------------------------
z_bit6_b	' cb 70
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		mov	tmp, #bit6
		jmp	#alu_bit
' -------------------------------------------------------
z_bit6_c	' cb 71
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		mov	tmp, #bit6
		jmp	#alu_bit
' -------------------------------------------------------
z_bit6_d	' cb 72
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		mov	tmp, #bit6
		jmp	#alu_bit
' -------------------------------------------------------
z_bit6_e	' cb 73
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		mov	tmp, #bit6
		jmp	#alu_bit
' -------------------------------------------------------
z_bit6_h	' cb 74
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		mov	tmp, #bit6
		jmp	#alu_bit
' -------------------------------------------------------
z_bit6_l	' cb 75
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		mov	tmp, #bit6
		jmp	#alu_bit
' -------------------------------------------------------
z_bit6_m	' cb 76
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, #bit6
		jmp	#alu_bit
' -------------------------------------------------------
z_bit6_a	' cb 77
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		mov	tmp, #bit6
		jmp	#alu_bit
' -------------------------------------------------------

' -------------------------------------------------------
z_bit7_b	' cb 78
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, B
		mov	tmp, #bit7
		jmp	#alu_bit
' -------------------------------------------------------
z_bit7_c	' cb 79
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, C
		mov	tmp, #bit7
		jmp	#alu_bit
' -------------------------------------------------------
z_bit7_d	' cb 7a
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, D
		mov	tmp, #bit7
		jmp	#alu_bit
' -------------------------------------------------------
z_bit7_e	' cb 7b
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, E
		mov	tmp, #bit7
		jmp	#alu_bit
' -------------------------------------------------------
z_bit7_h	' cb 7c
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, H
		mov	tmp, #bit7
		jmp	#alu_bit
' -------------------------------------------------------
z_bit7_l	' cb 7d
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, L
		mov	tmp, #bit7
		jmp	#alu_bit
' -------------------------------------------------------
z_bit7_m	' cb 7e
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, #bit7
		jmp	#alu_bit
' -------------------------------------------------------
z_bit7_a	' cb 7f
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, A
		mov	tmp, #bit7
		jmp	#alu_bit
' -------------------------------------------------------

' -------------------------------------------------------
z_res0_b	' cb 80
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	B, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_res0_c	' cb 81
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	C, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_res0_d	' cb 82
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	D, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_res0_e	' cb 83
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	E, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_res0_h	' cb 84
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	H, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_res0_l	' cb 85
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	L, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_res0_m	' cb 86
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		andn	alu, #bit0
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res0_a	' cb 87
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	A, #bit0
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_res1_b	' cb 88
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	B, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_res1_c	' cb 89
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	C, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_res1_d	' cb 8a
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	D, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_res1_e	' cb 8b
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	E, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_res1_h	' cb 8c
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	H, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_res1_l	' cb 8d
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	L, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_res1_m	' cb 8e
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		andn	alu, #bit1
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res1_a	' cb 8f
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	A, #bit1
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_res2_b	' cb 90
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	B, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_res2_c	' cb 91
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	C, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_res2_d	' cb 92
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	D, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_res2_e	' cb 93
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	E, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_res2_h	' cb 94
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	H, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_res2_l	' cb 95
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	L, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_res2_m	' cb 96
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		andn	alu, #bit2
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res2_a	' cb 97
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	A, #bit2
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_res3_b	' cb 98
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	B, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_res3_c	' cb 99
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	C, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_res3_d	' cb 9a
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	D, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_res3_e	' cb 9b
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	E, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_res3_h	' cb 9c
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	H, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_res3_l	' cb 9d
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	L, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_res3_m	' cb 9e
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		andn	alu, #bit3
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res3_a	' cb 9f
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	A, #bit3
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_res4_b	' cb a0
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	B, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_res4_c	' cb a1
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	C, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_res4_d	' cb a2
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	D, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_res4_e	' cb a3
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	E, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_res4_h	' cb a4
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	H, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_res4_l	' cb a5
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	L, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_res4_m	' cb a6
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		andn	alu, #bit4
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res4_a	' cb a7
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	A, #bit4
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_res5_b	' cb a8
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	B, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_res5_c	' cb a9
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	C, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_res5_d	' cb aa
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	D, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_res5_e	' cb ab
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	E, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_res5_h	' cb ac
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	H, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_res5_l	' cb ad
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	L, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_res5_m	' cb ae
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		andn	alu, #bit5
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res5_a	' cb af
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	A, #bit5
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_res6_b	' cb b0
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	B, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_res6_c	' cb b1
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	C, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_res6_d	' cb b2
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	D, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_res6_e	' cb b3
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	E, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_res6_h	' cb b4
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	H, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_res6_l	' cb b5
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	L, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_res6_m	' cb b6
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		andn	alu, #bit6
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res6_a	' cb b7
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	A, #bit6
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_res7_b	' cb b8
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	B, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_res7_c	' cb b9
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	C, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_res7_d	' cb ba
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	D, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_res7_e	' cb bb
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	E, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_res7_h	' cb bc
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	H, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_res7_l	' cb bd
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	L, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_res7_m	' cb be
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		andn	alu, #bit7
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res7_a	' cb bf
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	A, #bit7
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_set0_b	' cb c0
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	B, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_set0_c	' cb c1
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	C, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_set0_d	' cb c2
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	D, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_set0_e	' cb c3
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	E, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_set0_h	' cb c4
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	H, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_set0_l	' cb c5
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	L, #bit0
		jmp	#fetch
' -------------------------------------------------------
z_set0_m	' cb c6
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		or	alu, #bit0
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set0_a	' cb c7
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	A, #bit0
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_set1_b	' cb c8
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	B, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_set1_c	' cb c9
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	C, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_set1_d	' cb ca
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	D, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_set1_e	' cb cb
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	E, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_set1_h	' cb cc
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	H, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_set1_l	' cb cd
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	L, #bit1
		jmp	#fetch
' -------------------------------------------------------
z_set1_m	' cb ce
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		or	alu, #bit1
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set1_a	' cb cf
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	A, #bit1
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_set2_b	' cb d0
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	B, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_set2_c	' cb d1
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	C, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_set2_d	' cb d2
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	D, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_set2_e	' cb d3
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	E, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_set2_h	' cb d4
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	H, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_set2_l	' cb d5
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	L, #bit2
		jmp	#fetch
' -------------------------------------------------------
z_set2_m	' cb d6
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		or	alu, #bit2
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set2_a	' cb d7
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	A, #bit2
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_set3_b	' cb d8
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	B, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_set3_c	' cb d9
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	C, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_set3_d	' cb da
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	D, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_set3_e	' cb db
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	E, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_set3_h	' cb dc
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	H, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_set3_l	' cb dd
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	L, #bit3
		jmp	#fetch
' -------------------------------------------------------
z_set3_m	' cb de
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		or	alu, #bit3
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set3_a	' cb df
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	A, #bit3
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_set4_b	' cb e0
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	B, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_set4_c	' cb e1
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	C, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_set4_d	' cb e2
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	D, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_set4_e	' cb e3
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	E, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_set4_h	' cb e4
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	H, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_set4_l	' cb e5
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	L, #bit4
		jmp	#fetch
' -------------------------------------------------------
z_set4_m	' cb e6
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		or	alu, #bit4
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set4_a	' cb e7
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	A, #bit4
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_set5_b	' cb e8
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	B, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_set5_c	' cb e9
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	C, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_set5_d	' cb ea
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	D, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_set5_e	' cb eb
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	E, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_set5_h	' cb ec
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	H, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_set5_l	' cb ed
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	L, #bit5
		jmp	#fetch
' -------------------------------------------------------
z_set5_m	' cb ee
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		or	alu, #bit5
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set5_a	' cb ef
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	A, #bit5
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_set6_b	' cb f0
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	B, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_set6_c	' cb f1
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	C, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_set6_d	' cb f2
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	D, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_set6_e	' cb f3
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	E, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_set6_h	' cb f4
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	H, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_set6_l	' cb f5
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	L, #bit6
		jmp	#fetch
' -------------------------------------------------------
z_set6_m	' cb f6
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		or	alu, #bit6
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set6_a	' cb f7
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	A, #bit6
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_set7_b	' cb f8
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	B, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_set7_c	' cb f9
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	C, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_set7_d	' cb fa
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	D, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_set7_e	' cb fb
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	E, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_set7_h	' cb fc
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	H, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_set7_l	' cb fd
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	L, #bit7
		jmp	#fetch
' -------------------------------------------------------
z_set7_m	' cb fe
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		call	#rd_byte_hl
		or	alu, #bit7
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set7_a	' cb ff
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		or	A, #bit7
		jmp	#fetch
' -------------------------------------------------------

'******************************************************************************
'
'		OPCODES ED
'
'******************************************************************************

' -------------------------------------------------------
z_illegal
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
' fake opcode
z_bcde_cnt	' ed 00
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	alu, CNT
		mov	E, alu
		mov	D, alu
		and	E, #$ff
		shr	D, #8
		and	D, #$ff
		ror	alu, #8
		mov	C, alu
		mov	B, alu
		and	C, #$ff
		shr	B, #8
		and	B, #$ff
		jmp	#fetch
' -------------------------------------------------------
z_in_b_bc	' ed 40
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_port_bc
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_in_c_bc	' ed 48
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_port_bc
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------
z_in_d_bc	' ed 50
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_port_bc
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_in_e_bc	' ed 58
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_port_bc
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------
z_in_h_bc	' ed 60
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_port_bc
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_in_l_bc	' ed 68
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_port_bc
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_in_0_bc	' ed 70
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_port_bc
		jmp	#fetch
' -------------------------------------------------------
z_in_a_bc	' ed 78
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		call	#rd_port_bc
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_out_bc_b	' ed 41
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		mov	alu, B
		call	#wr_port_bc
		jmp	#fetch
' -------------------------------------------------------
z_out_bc_c	' ed 49
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		mov	alu, C
		call	#wr_port_bc
		jmp	#fetch
' -------------------------------------------------------
z_out_bc_d	' ed 51
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		mov	alu, D
		call	#wr_port_bc
		jmp	#fetch
' -------------------------------------------------------
z_out_bc_e	' ed 59
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		mov	alu, E
		call	#wr_port_bc
		jmp	#fetch
' -------------------------------------------------------
z_out_bc_h	' ed 61
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		mov	alu, H
		call	#wr_port_bc
		jmp	#fetch
' -------------------------------------------------------
z_out_bc_l	' ed 69
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		mov	alu, L
		call	#wr_port_bc
		jmp	#fetch
' -------------------------------------------------------
z_out_bc_0	' ed 71
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		mov	alu, #0
		call	#wr_port_bc
		jmp	#fetch
' -------------------------------------------------------
z_out_bc_a	' ed 79
#ifdef COUNT_C
		sub	cc, #12
#endif COUNT_C
		mov	alu, A
		call	#wr_port_bc
		jmp	#fetch
' -------------------------------------------------------


' -------------------------------------------------------
z_sbc_hl_bc	' ed 42
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		mov	tmp, B
		mov	t1, C
		jmp	#alu_sbc16
' -------------------------------------------------------
z_adc_hl_bc	' ed 4a
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		mov	tmp, B
		mov	t1, C
		jmp	#alu_adc16
' -------------------------------------------------------
z_sbc_hl_de	' ed 52
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		mov	tmp, D
		mov	t1, E
		jmp	#alu_sbc16
' -------------------------------------------------------
z_adc_hl_de	' ed 5a
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		mov	tmp, D
		mov	t1, E
		jmp	#alu_adc16
' -------------------------------------------------------
z_sbc_hl_hl	' ed 62
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		mov	tmp, H
		mov	t1, L
		jmp	#alu_sbc16
' -------------------------------------------------------
z_adc_hl_hl	' ed 6a
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		mov	tmp, H
		mov	t1, L
		jmp	#alu_adc16
' -------------------------------------------------------
z_sbc_hl_sp	' ed 72
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		mov	tmp, SP
		mov	t1, SP
		shr	tmp, #8				' MSB of SP
		and	t1, #$ff			' LSB of SP
		jmp	#alu_sbc16
' -------------------------------------------------------
z_adc_hl_sp	' ed 7a
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		mov	tmp, SP
		mov	t1, SP
		shr	tmp, #8				' MSB of SP
		and	t1, #$ff			' LSB of SP
		jmp	#alu_adc16
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_abs16_bc	' ed 43
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		mov	alu, B
		shl	alu, #8
		or	alu, C
		call	#wr_word
		jmp	#fetch
' -------------------------------------------------------
z_ld_bc_abs16	' ed 4b
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		call	#rd_word
		mov	C, alu
		mov	B, alu
		and	C, #$ff
		shr	B, #8
		jmp	#fetch
' -------------------------------------------------------
z_ld_abs16_de	' ed 53
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		mov	alu, D
		shl	alu, #8
		or	alu, E
		call	#wr_word
		jmp	#fetch
' -------------------------------------------------------
z_ld_de_abs16	' ed 5b
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		call	#rd_word
		mov	E, alu
		mov	D, alu
		and	E, #$ff
		shr	D, #8
		jmp	#fetch
' -------------------------------------------------------
z_ld_abs16_hl_2	' ed 63
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		mov	alu, H
		shl	alu, #8
		or	alu, L
		call	#wr_word
		jmp	#fetch
' -------------------------------------------------------
z_ld_hl_abs16_2	' ed 6b
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		call	#rd_word
		mov	L, alu
		mov	H, alu
		and	L, #$ff
		shr	H, #8
		jmp	#fetch
' -------------------------------------------------------
z_ld_abs16_sp	' ed 73
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		mov	alu, SP
		call	#wr_word
		jmp	#fetch
' -------------------------------------------------------
z_ld_sp_abs16	' ed 7b
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		call	#rd_word
		mov	SP, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_neg		' ed 44 (4c, 54, 5c, 64, 6c, 74, 7c)
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		mov	tmp, A
		mov	A, #0
		jmp	#alu_sub
' -------------------------------------------------------
z_retn		' ed 45
#ifdef COUNT_C
		sub	cc, #14
#endif COUNT_C
		test	IFF, #IFF2	WZ		' copy IFF2
		muxnz	IFF, #IFF1			' to IFF1
		jmp	#do_ret
' -------------------------------------------------------
z_reti		' ed 4d
#ifdef COUNT_C
		sub	cc, #14
#endif COUNT_C
		test	IFF, #IFF2	WZ		' copy IFF2
		muxnz	IFF, #IFF1			' to IFF1
		' TODO: RETI devices daisy chain handling
		jmp	#do_ret
' -------------------------------------------------------
z_im_0		' ed 46 (4e, 66, 6e)
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	IFF, #(IM1|IM2)			' reset interrupt mode bits
		jmp	#fetch
' -------------------------------------------------------
z_im_1		' ed 56 (76)
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	IFF, #IM2			' reset interrupt mode bit 2
		or	IFF, #IM1			' set interrupt mode 1
		jmp	#fetch
' -------------------------------------------------------
z_im_2		' ed 5e (7e)
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		andn	IFF, #IM1			' reset interrupt mode bit 1
		or	IFF, #IM2			' set interrupt mode 2
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_i_a	' ed 47
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		wrbyte	A, I
		jmp	#fetch
' -------------------------------------------------------
z_ld_r_a	' ed 4f
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		mov	R, A
		mov	R2, A
		and	R2, #$80
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_i	' ed 57
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	A, I
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_r	' ed 5f
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		mov	A, R
		and	A, #$7f
		or	A, R2
		jmp	#fetch
' -------------------------------------------------------
z_rrd_m		' ed 67
' ed 67 Rotate right digit memory (HL)
'
' A:[7][6][5][4][3][2][1][0] M:[7][6][5][4][3][2][1][0]
' before         x  x  x  x     y  y  y  y  z  z  z  z
' after          z  z  z  z     x  x  x  x  y  y  y  y
'
#ifdef COUNT_C
		sub	cc, #18
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		shr	tmp, #4				' upper nibble to 3..0
		mov	aux, A
		shl	aux, #4				' A to bits 11..4
		and	aux, #$f0			' mask bits 7..4
		or	tmp, aux			' combine with lower nibble
		and	alu, #$0f			' mask 3..0 of memory
		andn	A, #%0000_1111			' clears bits 3..0 in accu
		or	A, alu				' combine with accu
		mov	alu, tmp			' write back memory
		call	#wr_byte
		and	F, #CF				' keep C flag
		and	A, #$ff		WZ, WC		' get zero and parity for A
		muxz	F, #ZF
		muxnc	F, #PF
		test	A, #$80		WZ		' get sign for A
		muxnz	F, #SF
		jmp	#fetch
' -------------------------------------------------------
z_rld_m		' ed 67
' ed 6f Rotate left digit memory (HL)
'
' A:[7][6][5][4][3][2][1][0] M:[7][6][5][4][3][2][1][0]
' before         x  x  x  x     y  y  y  y  z  z  z  z
' after          y  y  y  y     z  z  z  z  x  x  x  x
'
#ifdef COUNT_C
		sub	cc, #18
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		shl	tmp, #4				' mem to bits 11..4
		mov	aux, A
		and	aux, #$0f			' mask bits 3..0 of accu
		or	tmp, aux			' combine with upper nibble
		shr	alu, #4				' mem upper nibble to 3..0
		andn	A, #%0000_1111			' clear bits 3..0 in accu
		or	A, alu				' combine with accu
		mov	alu, tmp			' write back memory
		and	alu, #$ff			' needed?
		call	#wr_byte
		and	F, #CF				' keep C flag
		and	A, #$ff		WZ, WC		' get zero and parity for A
		muxz	F, #ZF
		muxnc	F, #PF
		test	A, #$80		WZ		' get sign for A
		muxnz	F, #SF
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ldi		' ed a0
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_byte_hl
		call	#k_inc_hl
		call	#wr_byte_de
		call	#k_inc_de
		call	#k_dec_bc
		and	F, #(SF | ZF | CF)
	if_nz	or	F, #PF
		jmp	#fetch
' -------------------------------------------------------
z_ldd		' ed a8
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_byte_hl
		call	#k_dec_hl
		call	#wr_byte_de
		call	#k_dec_de
		call	#k_dec_bc
		and	F, #(SF | ZF | CF)
	if_nz	or	F, #PF
		jmp	#fetch
' -------------------------------------------------------
z_ldir		' ed b0
#ifdef COUNT_R
		sub	R, #1
#endif COUNT_R
:loop
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
#ifdef COUNT_R
		add	R, #1
#endif COUNT_R
		call	#rd_byte_hl
		call	#k_inc_hl
		call	#wr_byte_de
		call	#k_inc_de
		call	#k_dec_bc
	if_nz	sub	lmm_pc, #4*($ + 1 - :loop)
		and	F, #(SF | ZF | CF)
		jmp	#fetch
' -------------------------------------------------------
z_lddr		' ed b8
#ifdef COUNT_R
		sub	R, #1
#endif COUNT_R
:loop
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
#ifdef COUNT_R
		add	R, #1
#endif COUNT_R
		call	#rd_byte_hl
		call	#k_dec_hl
		call	#wr_byte_de
		call	#k_dec_de
		call	#k_dec_bc
	if_nz	sub	lmm_pc, #4*($ + 1 - :loop)
		and	F, #(SF | ZF | CF)
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_cpi		' ed a1
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		and	F, #CF				' keep carry flag
		or	F, #NF				' always set negative flag
		mov	alu, A
		mov     aux, alu			' keep a copy in aux for later
		sub     alu, tmp	WZ
		muxz    F, #ZF				' set zero flag if SUB was zero
		xor     aux, alu
		xor     aux, tmp
		and     aux, #HF			' mask the auxiliary flag
		or      F, aux
		test    alu, #$80	WZ
		muxnz   F, #SF				' set sign flag if alu bit #7 is set
		call	#k_inc_hl			' HL++
		call	#k_dec_bc			' BC-- and set zero if BC == 0
		muxnz   F, #PF				' set parity flag if BC was non zero
		jmp	#fetch
' -------------------------------------------------------
z_cpd		' ed a9
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		and	F, #CF				' keep carry flag
		or	F, #NF				' always set negative flag
		mov	alu, A
		mov     aux, alu			' keep a copy in aux for later
		sub     alu, tmp	WZ
		muxz    F, #ZF				' set zero flag if SUB was zero
		xor     aux, alu
		xor     aux, tmp
		and     aux, #HF			' mask the auxiliary flag
		or      F, aux
		test    alu, #$80	WZ
		muxnz   F, #SF				' set sign flag if alu bit #7 is set
		call	#k_dec_hl			' HL--
		call	#k_dec_bc			' BC-- and set zero if BC == 0
		muxnz   F, #PF				' set parity flag if BC was non zero
		jmp	#fetch
' -------------------------------------------------------
z_cpir		' ed b1
#ifdef COUNT_R
		sub	R, #1
#endif COUNT_R
:loop
#ifdef COUNT_R
		add	R, #1
#endif COUNT_R
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		mov	alu, A
		mov	aux, alu
		sub	alu, tmp	WZ
		call	#k_inc_hl			' HL++
		sub	C, #1		WC		' BC--
		subx	B, #0
		and	C, #$ff
		and	B, #$ff
	if_nz   mov	t1, C				' if alu - tmp is non zero, is BC zero now?
	if_nz	or	t1, B		WZ
	if_nz   sub	lmm_pc, #4*($ + 1 - :loop)	' loop if both are non zero
		and     F, #CF				' keep carry flag
		or      F, #NF				' always set the negative flag
		xor     aux, alu
		xor     aux, tmp
		and     aux, #HF			' mask the auxiliary flag
		or      F, aux
		and     alu, #$ff	WZ
		muxz    F, #ZF				' set zero flag if alu is zero
		test    alu, #$80	WZ
		muxnz   F, #SF				' set sign flag if alu bit #7 is set
		mov	t1, C				' check BC == 0
		or	t1, B		WZ
		muxnz   F, #PF				' set P flag if non zero
		jmp	#fetch
' -------------------------------------------------------
z_cpdr		' ed b9
#ifdef COUNT_R
		sub	R, #1
#endif COUNT_R
:loop
#ifdef COUNT_R
		add	R, #1
#endif COUNT_R
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_byte_hl
		mov	tmp, alu
		mov	alu, A
		mov	aux, alu
		sub	alu, tmp	WZ
		call	#k_dec_hl			' HL--
		sub	C, #1		WC		' BC--
		subx	B, #0
		and	C, #$ff
		and	B, #$ff
	if_nz   mov	t1, C				' if alu - tmp is non zero, is BC zero now?
	if_nz	or	t1, B		WZ
	if_nz   sub	lmm_pc, #4*($ + 1 - :loop)	' loop if both are non zero
		and     F, #CF				' keep carry flag
		or      F, #NF				' always set the negative flag
		xor     aux, alu
		xor     aux, tmp
		and     aux, #HF			' mask the auxiliary flag
		or      F, aux
		and     alu, #$ff	WZ
		muxz    F, #ZF				' set zero flag if alu is zero
		test    alu, #$80	WZ
		muxnz   F, #SF				' set sign flag if alu bit #7 is set
		mov	t1, C				' check BC == 0
		or	t1, B		WZ
		muxnz   F, #PF				' set P flag if non zero
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ini		' ed a2
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_port_bc
		mov	ea, H
		shl	ea, #8
		or	ea, L
		call	#wr_byte
		call	#k_inc_hl
		sub	B, #1		WZ		' B--
		muxz	F, #ZF
		test	B, #$80		WC
		muxc	F, #SF
		jmp	#fetch
' -------------------------------------------------------
z_ind		' ed aa
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_port_bc
		mov	ea, H
		shl	ea, #8
		or	ea, L
		call	#wr_byte
		call	#k_dec_hl
		sub	B, #1		WZ		' B--
		muxz	F, #ZF
		test	B, #$80		WC
		muxc	F, #SF
		jmp	#fetch
' -------------------------------------------------------
z_inir		' ed b2
:loop
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_port_bc
		sub	B, #1		WZ		' B--
		mov	ea, H
		shl	ea, #1
		or	ea, L
		call	#wr_byte
		call	#k_inc_hl			' HL++
	if_nz	sub	lmm_pc, #4*($ + 1 - :loop)	' until B == 0
		andn	F, #SF
		or	F, #ZF
		jmp	#fetch
' -------------------------------------------------------
z_indr		' ed ba
:loop
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_port_bc
		sub	B, #1		WZ		' B--
		mov	ea, H
		shl	ea, #1
		or	ea, L
		call	#wr_byte
		call	#k_dec_hl			' HL--
	if_nz	sub	lmm_pc, #4*($ + 1 - :loop)	' until B == 0
		andn	F, #SF
		or	F, #ZF
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_outi		' ed a3
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_byte_hl
		call	#k_inc_hl			' HL++
		call	#wr_port_bc
		sub	B, #1		WZ		' B--
		muxz	F, #ZF
		test	B, #$80		WZ
		muxnz	F, #SF
		jmp	#fetch
' -------------------------------------------------------
z_outd		' ed ab
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_byte_hl
		call	#k_dec_hl			' HL--
		call	#wr_port_bc
		sub	B, #1		WZ		' B--
		muxz	F, #ZF
		test	B, #$80		WZ
		muxnz	F, #SF
		jmp	#fetch
' -------------------------------------------------------
z_otir		' ed b3
:loop
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_byte_hl
		call	#k_inc_hl			' HL++
		call	#wr_port_bc
		sub	B, #1		WZ		' B--
	if_nz	sub	lmm_pc, #4*($ + 1 - :loop)
		andn	F, #SF				' clear sign flag
		or	F, #ZF				' set zero flag
		jmp	#fetch
' -------------------------------------------------------
z_otdr		' ed bb
:loop
#ifdef COUNT_C
		sub	cc, #16
#endif COUNT_C
		call	#rd_byte_hl
		call	#k_dec_hl			' HL--
		call	#wr_port_bc
		sub	B, #1		WZ		' B--
	if_nz	sub	lmm_pc, #4*($ + 1 - :loop)
		andn	F, #SF				' clear sign flag
		or	F, #ZF				' set zero flag
		jmp	#fetch
' -------------------------------------------------------

'******************************************************************************
'
'		OPCODES DD/FD
'
'******************************************************************************

' -------------------------------------------------------
z_add_xy_bc	' dd/fd 09
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		mov	tmp, B
		shl	tmp, #8
		or	tmp, C
		jmp	#alu_add16_xy
' -------------------------------------------------------
z_add_xy_de	' dd/fd 19
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		mov	tmp, D
		shl	tmp, #8
		or	tmp, E
		jmp	#alu_add16_xy
' -------------------------------------------------------
z_add_xy_xy	' dd/fd 29
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		rdword	tmp, XY
		jmp	#alu_add16_xy
' -------------------------------------------------------
z_add_xy_sp	' dd/fd 39
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		mov	tmp, SP
		jmp	#alu_add16_xy
' -------------------------------------------------------
z_ld_xy_imm16	' dd/fd 21
#ifdef COUNT_C
		sub	cc, #14
#endif COUNT_C
		call	#rd_opword
		wrword	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_abs16_xy	' dd/fd 22
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		rdword	alu, XY
		call	#wr_word
		jmp	#fetch
' -------------------------------------------------------
z_ld_xy_abs16	' dd/fd 2a
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		call	#rd_opword
		mov	ea, alu
		call	#rd_word
		wrword	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_inc_xy	' dd/fd 23
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		rdword	alu, XY
		add	alu, #1
		wrword	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_dec_xy	' dd/fd 2b
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		rdword	alu, XY
		sub	alu, #1
		wrword	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_inc_hxy	' dd/fd 24
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	alu, XY
		call	#alu_inc
		wrbyte	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_dec_hxy	' dd/fd 25
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	alu, XY
		call	#alu_dec
		wrbyte	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_hxy_imm8	' dd/fd 26
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		add	XY, #1
		call	#rd_opcode
		wrbyte	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_inc_lxy	' dd/fd 2c
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	alu, XY
		call	#alu_inc
		wrbyte	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_dec_lxy	' dd/fd 2d
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	alu, XY
		call	#alu_dec
		wrbyte	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_lxy_imm8	' dd/fd 2e
#ifdef COUNT_C
		sub	cc, #11
#endif COUNT_C
		call	#rd_opcode
		wrbyte	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_inc_mxy	' dd/fd 34
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		call	#alu_inc
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_dec_mxy	' dd/fd 35
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		call	#alu_dec
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_mxy_imm8	' dd/fd 36
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		mov	tmp, ea
		call	#rd_opcode
		mov	ea, tmp
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_b_hxy	' dd/fd 44
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	B, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_b_lxy	' dd/fd 45
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	B, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_b_mxy	' dd/fd 46
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	B, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_c_hxy	' dd/fd 4c
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	C, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_c_lxy	' dd/fd 4d
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	C, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_c_mxy	' dd/fd 4e
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	C, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_d_hxy	' dd/fd 54
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	D, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_d_lxy	' dd/fd 55
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	D, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_d_mxy	' dd/fd 56
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	D, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_e_hxy	' dd/fd 5c
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	E, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_e_lxy	' dd/fd 5d
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	E, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_e_mxy	' dd/fd 5e
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	E, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_hxy_b	' dd/fd 60
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		wrbyte	B, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_hxy_c	' dd/fd 61
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		wrbyte	C, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_hxy_d	' dd/fd 62
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		wrbyte	D, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_hxy_e	' dd/fd 63
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		wrbyte	E, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_hxy_hxy	' dd/fd 64
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_ld_hxy_lxy	' dd/fd 65
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	alu, XY
		add	XY, #1
		wrbyte	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_h_mxy	' dd/fd 66
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	H, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_hxy_a	' dd/fd 67
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		wrbyte	A, XY
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_lxy_b	' dd/fd 68
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		wrbyte	B, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_lxy_c	' dd/fd 69
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		wrbyte	C, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_lxy_d	' dd/fd 6a
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		wrbyte	D, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_lxy_e	' dd/fd 6b
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		wrbyte	E, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_lxy_hxy	' dd/fd 6c
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	alu, XY
		sub	XY, #1
		wrbyte	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_lxy_lxy	' dd/fd 6d
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		jmp	#fetch
' -------------------------------------------------------
z_ld_l_mxy	' dd/fd 6e
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	L, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_lxy_a	' dd/fd 6f
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		wrbyte	A, XY
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_ld_mxy_b	' dd/fd 70
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		mov	alu, B
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_mxy_c	' dd/fd 71
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		mov	alu, C
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_mxy_d	' dd/fd 72
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		mov	alu, D
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_mxy_e	' dd/fd 73
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		mov	alu, E
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_mxy_h	' dd/fd 74
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		mov	alu, H
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_mxy_l	' dd/fd 75
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		mov	alu, L
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_mxy_a	' dd/fd 77
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		mov	alu, A
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_hxy	' dd/fd 7c
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	alu, XY
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_lxy	' dd/fd 7d
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	alu, XY
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------
z_ld_a_mxy	' dd/fd 7e
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	A, alu
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_add_a_hxy	' dd/fd 84
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	tmp, XY
		jmp	#alu_add
' -------------------------------------------------------
z_add_a_lxy	' dd/fd 85
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	tmp, XY
		jmp	#alu_add
' -------------------------------------------------------
z_add_a_mxy	' dd/fd 86
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	tmp, alu
		jmp	#alu_add
' -------------------------------------------------------

' -------------------------------------------------------
z_adc_a_hxy	' dd/fd 8c
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	tmp, XY
		jmp	#alu_adc
' -------------------------------------------------------
z_adc_a_lxy	' dd/fd 8d
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	tmp, XY
		jmp	#alu_adc
' -------------------------------------------------------
z_adc_a_mxy	' dd/fd 8e
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	tmp, alu
		jmp	#alu_adc
' -------------------------------------------------------

' -------------------------------------------------------
z_sub_hxy	' dd/fd 94
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	tmp, XY
		jmp	#alu_sub
' -------------------------------------------------------
z_sub_lxy	' dd/fd 95
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	tmp, XY
		jmp	#alu_sub
' -------------------------------------------------------
z_sub_mxy	' dd/fd 96
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	tmp, alu
		jmp	#alu_sub
' -------------------------------------------------------

' -------------------------------------------------------
z_sbc_a_hxy	' dd/fd 9c
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	tmp, XY
		jmp	#alu_sbc
' -------------------------------------------------------
z_sbc_a_lxy	' dd/fd 9d
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	tmp, XY
		jmp	#alu_sbc
' -------------------------------------------------------
z_sbc_a_mxy	' dd/fd 9e
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	tmp, alu
		jmp	#alu_sbc
' -------------------------------------------------------

' -------------------------------------------------------
z_and_hxy	' dd/fd a4
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	tmp, XY
		jmp	#alu_and
' -------------------------------------------------------
z_and_lxy	' dd/fd a5
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	tmp, XY
		jmp	#alu_and
' -------------------------------------------------------
z_and_mxy	' dd/fd a6
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	tmp, alu
		jmp	#alu_and
' -------------------------------------------------------

' -------------------------------------------------------
z_xor_hxy	' dd/fd ac
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	tmp, XY
		jmp	#alu_xor
' -------------------------------------------------------
z_xor_lxy	' dd/fd ad
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	tmp, XY
		jmp	#alu_xor
' -------------------------------------------------------
z_xor_mxy	' dd/fd ae
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	tmp, alu
		jmp	#alu_xor
' -------------------------------------------------------

' -------------------------------------------------------
z_or_hxy	' dd/fd b4
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	tmp, XY
		jmp	#alu_or
' -------------------------------------------------------
z_or_lxy	' dd/fd b5
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	tmp, XY
		jmp	#alu_or
' -------------------------------------------------------
z_or_mxy	' dd/fd b6
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	tmp, alu
		jmp	#alu_or
' -------------------------------------------------------

' -------------------------------------------------------
z_cp_hxy	' dd/fd bc
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		add	XY, #1
		rdbyte	tmp, XY
		jmp	#alu_cp
' -------------------------------------------------------
z_cp_lxy	' dd/fd bd
#ifdef COUNT_C
		sub	cc, #9
#endif COUNT_C
		rdbyte	tmp, XY
		jmp	#alu_cp
' -------------------------------------------------------
z_cp_mxy	' dd/fd be
#ifdef COUNT_C
		sub	cc, #19
#endif COUNT_C
		call	#mem_xy
		call	#rd_byte
		mov	tmp, alu
		jmp	#alu_cp
' -------------------------------------------------------

' -------------------------------------------------------
z_xy_cb_xx	' dd/fd cb
		call	#mem_xy
		mov	aux, ea				' save address
		call	#rd_opcode			' 4th byte
		shr	alu, #3				' / 8
		mov	tmp, table_xy_cb		' short table of 32 entries
		add	tmp, alu
		add	tmp, alu
		mov	ea, aux				' restore address (IX/IY+disp8)
		call	#rd_byte			' read the byte there
		jmp	#fetch2
' -------------------------------------------------------

' -------------------------------------------------------
z_pop_xy	' dd/fd e1
#ifdef COUNT_C
		sub	cc, #14
#endif COUNT_C
		call	#pop_alu
		wrword	alu, XY
		jmp	#fetch
' -------------------------------------------------------
z_ex_sp_xy	' dd/fd e3
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		mov	ea, SP
		call	#rd_word
		rdword	tmp, XY
		xor	alu, tmp
		xor	tmp, alu
		xor	alu, tmp
		wrword	tmp, XY
		call	#wr_word
		jmp	#fetch
' -------------------------------------------------------
z_push_xy	' dd/fd e5
#ifdef COUNT_C
		sub	cc, #15
#endif COUNT_C
		rdword	alu, XY
		call	#push_alu
		jmp	#fetch
' -------------------------------------------------------
z_jp_xy		' dd/fd e9
#ifdef COUNT_C
		sub	cc, #8
#endif COUNT_C
		rdword	PC, XY
		jmp	#fetch
' -------------------------------------------------------
z_ld_sp_xy	' dd/fd f9
#ifdef COUNT_C
		sub	cc, #10
#endif COUNT_C
		rdword	SP, XY
		jmp	#fetch
' -------------------------------------------------------

'******************************************************************************
'
'		OPCODES DD/FD CB
'
'******************************************************************************

' -------------------------------------------------------
z_rlc_mxy	' dd/fd cb 00..07
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		call	#alu_rlc
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_rrc_mxy	' dd/fd cb 08..0f
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		call	#alu_rrc
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_rl_mxy	' dd/fd cb 10..17
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		call	#alu_rl
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_rr_mxy	' dd/fd cb 18..1f
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		call	#alu_rr
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_sla_mxy	' dd/fd cb 20..27
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		call	#alu_sla
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_sra_mxy	' dd/fd cb 28..2f
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		call	#alu_sra
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_sli_mxy	' dd/fd cb 30..37
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		call	#alu_sli
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_srl_mxy	' dd/fd cb 38..3f
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		call	#alu_srl
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_bit0_mxy	' dd/fd cb 40..47
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		mov	tmp, #bit0
		jmp	#alu_bit
' -------------------------------------------------------
z_bit1_mxy	' dd/fd cb 48..4f
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		mov	tmp, #bit1
		jmp	#alu_bit
' -------------------------------------------------------
z_bit2_mxy	' dd/fd cb 50..57
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		mov	tmp, #bit2
		jmp	#alu_bit
' -------------------------------------------------------
z_bit3_mxy	' dd/fd cb 58..5f
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		mov	tmp, #bit3
		jmp	#alu_bit
' -------------------------------------------------------
z_bit4_mxy	' dd/fd cb 60..67
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		mov	tmp, #bit4
		jmp	#alu_bit
' -------------------------------------------------------
z_bit5_mxy	' dd/fd cb 68..6f
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		mov	tmp, #bit5
		jmp	#alu_bit
' -------------------------------------------------------
z_bit6_mxy	' dd/fd cb 70..77
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		mov	tmp, #bit6
		jmp	#alu_bit
' -------------------------------------------------------
z_bit7_mxy	' dd/fd cb 78..7f
#ifdef COUNT_C
		sub	cc, #20
#endif COUNT_C
		mov	tmp, #bit7
		jmp	#alu_bit
' -------------------------------------------------------

' -------------------------------------------------------
z_res0_mxy	' dd/fd cb 80..87
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		andn	alu, #bit0
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res1_mxy	' dd/fd cb 88..8f
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		andn	alu, #bit1
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res2_mxy	' dd/fd cb 90..97
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		andn	alu, #bit2
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res3_mxy	' dd/fd cb 98..9f
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		andn	alu, #bit3
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res4_mxy	' dd/fd cb a0..a7
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		andn	alu, #bit4
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res5_mxy	' dd/fd cb a8..af
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		andn	alu, #bit5
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res6_mxy	' dd/fd cb b0..b7
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		andn	alu, #bit6
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_res7_mxy	' dd/fd cb b8..bf
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		andn	alu, #bit7
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------

' -------------------------------------------------------
z_set0_mxy	' dd/fd cb c0..c7
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		or	alu, #bit0
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set1_mxy	' dd/fd cb c8..cf
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		or	alu, #bit1
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set2_mxy	' dd/fd cb d0..d7
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		or	alu, #bit2
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set3_mxy	' dd/fd cb d8..df
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		or	alu, #bit3
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set4_mxy	' dd/fd cb e0..e7
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		or	alu, #bit4
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set5_mxy	' dd/fd cb e8..ef
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		or	alu, #bit5
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set6_mxy	' dd/fd cb f0..f7
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		or	alu, #bit6
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
z_set7_mxy	' dd/fd cb f8..ff
#ifdef COUNT_C
		sub	cc, #23
#endif COUNT_C
		or	alu, #bit7
		call	#wr_byte
		jmp	#fetch
' -------------------------------------------------------
#ifdef INTERRUPTS
do_int
		mov	tmp, tmp	WC		' is this an NMI request?
	if_c	add	lmm_pc, (:nmi - $ - 1)		' LMM jump to the NMI handler
		test	IFF, #IFF1	WZ		' interrupts enabled?
	if_z	jmp	#fetch0				' no, just fetch next opcode
		andn	IFF, #(IFF1|IFF2)		' disable further interrupts
		and	tmp, #%00111000			' expecting a restart vector in int_request
		mov	alu, PC
		call	#push_alu
		mov	PC, tmp
		mov	tmp, #0
		wrlong	tmp, int_request		' clear the interrupt request
		jmp	#fetch
:nmi
		andn	IFF, #IFF1			' clear IFF1 - may be re-enabled by RETN or RETI
		call	#push_alu
		mov	PC, tmp
		mov	PC, #$0066			' NMI vector
		mov	tmp, #0
		wrlong	tmp, int_request		' clear the interrupt request
		jmp	#fetch
#endif INTERRUPTS
' -------------------------------------------------------

opcodes_00
	{00}	word	@@@z_nop
	{01}	word	@@@z_ld_bc_imm16
	{02}	word	@@@z_ld_mem_bc_a
	{03}	word	@@@z_inc_bc
	{04}	word	@@@z_inc_b
	{05}	word	@@@z_dec_b
	{06}	word	@@@z_ld_b_imm8
	{07}	word	@@@z_rlca

	{08}	word	@@@z_ex_af_af2
	{09}	word	@@@z_add_hl_bc
	{0a}	word	@@@z_ld_a_mem_bc
	{0b}	word	@@@z_dec_bc
	{0c}	word	@@@z_inc_c
	{0d}	word	@@@z_dec_c
	{0e}	word	@@@z_ld_c_imm8
	{0f}	word	@@@z_rrca

	{10}	word	@@@z_djnz
	{11}	word	@@@z_ld_de_imm16
	{12}	word	@@@z_ld_mem_de_a
	{13}	word	@@@z_inc_de
	{14}	word	@@@z_inc_d
	{15}	word	@@@z_dec_d
	{16}	word	@@@z_ld_d_imm8
	{17}	word	@@@z_rla

	{18}	word	@@@z_jr
	{19}	word	@@@z_add_hl_de
	{1a}	word	@@@z_ld_a_mem_de
	{1b}	word	@@@z_dec_de
	{1c}	word	@@@z_inc_e
	{1d}	word	@@@z_dec_e
	{1e}	word	@@@z_ld_e_imm8
	{1f}	word	@@@z_rra

	{20}	word	@@@z_jr_nz
	{21}	word	@@@z_ld_hl_imm16
	{22}	word	@@@z_ld_abs16_hl
	{23}	word	@@@z_inc_hl
	{24}	word	@@@z_inc_h
	{25}	word	@@@z_dec_h
	{26}	word	@@@z_ld_h_imm8
	{27}	word	@@@z_daa

	{28}	word	@@@z_jr_z
	{29}	word	@@@z_add_hl_hl
	{2a}	word	@@@z_ld_hl_abs16
	{2b}	word	@@@z_dec_hl
	{2c}	word	@@@z_inc_l
	{2d}	word	@@@z_dec_l
	{2e}	word	@@@z_ld_l_imm8
	{2f}	word	@@@z_cpl

	{30}	word	@@@z_jr_nc
	{31}	word	@@@z_ld_sp_imm16
	{32}	word	@@@z_ld_abs16_a
	{33}	word	@@@z_inc_sp
	{34}	word	@@@z_inc_m
	{35}	word	@@@z_dec_m
	{36}	word	@@@z_ld_m_imm8
	{37}	word	@@@z_scf

	{38}	word	@@@z_jr_c
	{39}	word	@@@z_add_hl_sp
	{3a}	word	@@@z_ld_a_abs16
	{3b}	word	@@@z_dec_sp
	{3c}	word	@@@z_inc_a
	{3d}	word	@@@z_dec_a
	{3e}	word	@@@z_ld_a_imm8
	{3f}	word	@@@z_ccf

	{40}	word	@@@z_ld_b_b
	{41}	word	@@@z_ld_b_c
	{42}	word	@@@z_ld_b_d
	{43}	word	@@@z_ld_b_e
	{44}	word	@@@z_ld_b_h
	{45}	word	@@@z_ld_b_l
	{46}	word	@@@z_ld_b_m
	{47}	word	@@@z_ld_b_a

	{48}	word	@@@z_ld_c_b
	{49}	word	@@@z_ld_c_c
	{4a}	word	@@@z_ld_c_d
	{4b}	word	@@@z_ld_c_e
	{4c}	word	@@@z_ld_c_h
	{4d}	word	@@@z_ld_c_l
	{4e}	word	@@@z_ld_c_m
	{4f}	word	@@@z_ld_c_a

	{50}	word	@@@z_ld_d_b
	{51}	word	@@@z_ld_d_c
	{52}	word	@@@z_ld_d_d
	{53}	word	@@@z_ld_d_e
	{54}	word	@@@z_ld_d_h
	{55}	word	@@@z_ld_d_l
	{56}	word	@@@z_ld_d_m
	{57}	word	@@@z_ld_d_a

	{58}	word	@@@z_ld_e_b
	{59}	word	@@@z_ld_e_c
	{5a}	word	@@@z_ld_e_d
	{5b}	word	@@@z_ld_e_e
	{5c}	word	@@@z_ld_e_h
	{5d}	word	@@@z_ld_e_l
	{5e}	word	@@@z_ld_e_m
	{5f}	word	@@@z_ld_e_a

	{60}	word	@@@z_ld_h_b
	{61}	word	@@@z_ld_h_c
	{62}	word	@@@z_ld_h_d
	{63}	word	@@@z_ld_h_e
	{64}	word	@@@z_ld_h_h
	{65}	word	@@@z_ld_h_l
	{66}	word	@@@z_ld_h_m
	{67}	word	@@@z_ld_h_a

	{68}	word	@@@z_ld_l_b
	{69}	word	@@@z_ld_l_c
	{6a}	word	@@@z_ld_l_d
	{6b}	word	@@@z_ld_l_e
	{6c}	word	@@@z_ld_l_h
	{6d}	word	@@@z_ld_l_l
	{6e}	word	@@@z_ld_l_m
	{6f}	word	@@@z_ld_l_a

	{70}	word	@@@z_ld_m_b
	{71}	word	@@@z_ld_m_c
	{72}	word	@@@z_ld_m_d
	{73}	word	@@@z_ld_m_e
	{74}	word	@@@z_ld_m_h
	{75}	word	@@@z_ld_m_l
	{76}	word	@@@z_halt
	{77}	word	@@@z_ld_m_a

	{78}	word	@@@z_ld_a_b
	{79}	word	@@@z_ld_a_c
	{7a}	word	@@@z_ld_a_d
	{7b}	word	@@@z_ld_a_e
	{7c}	word	@@@z_ld_a_h
	{7d}	word	@@@z_ld_a_l
	{7e}	word	@@@z_ld_a_m
	{7f}	word	@@@z_ld_a_a

	{80}	word	@@@z_add_a_b
	{81}	word	@@@z_add_a_c
	{82}	word	@@@z_add_a_d
	{83}	word	@@@z_add_a_e
	{84}	word	@@@z_add_a_h
	{85}	word	@@@z_add_a_l
	{86}	word	@@@z_add_a_m
	{87}	word	@@@z_add_a_a

	{88}	word	@@@z_adc_a_b
	{89}	word	@@@z_adc_a_c
	{8a}	word	@@@z_adc_a_d
	{8b}	word	@@@z_adc_a_e
	{8c}	word	@@@z_adc_a_h
	{8d}	word	@@@z_adc_a_l
	{8e}	word	@@@z_adc_a_m
	{8f}	word	@@@z_adc_a_a

	{90}	word	@@@z_sub_b
	{91}	word	@@@z_sub_c
	{92}	word	@@@z_sub_d
	{93}	word	@@@z_sub_e
	{94}	word	@@@z_sub_h
	{95}	word	@@@z_sub_l
	{96}	word	@@@z_sub_m
	{97}	word	@@@z_sub_a

	{98}	word	@@@z_sbc_a_b
	{99}	word	@@@z_sbc_a_c
	{9a}	word	@@@z_sbc_a_d
	{9b}	word	@@@z_sbc_a_e
	{9c}	word	@@@z_sbc_a_h
	{9d}	word	@@@z_sbc_a_l
	{9e}	word	@@@z_sbc_a_m
	{9f}	word	@@@z_sbc_a_a

	{a0}	word	@@@z_and_b
	{a1}	word	@@@z_and_c
	{a2}	word	@@@z_and_d
	{a3}	word	@@@z_and_e
	{a4}	word	@@@z_and_h
	{a5}	word	@@@z_and_l
	{a6}	word	@@@z_and_m
	{a7}	word	@@@z_and_a

	{a8}	word	@@@z_xor_b
	{a9}	word	@@@z_xor_c
	{aa}	word	@@@z_xor_d
	{ab}	word	@@@z_xor_e
	{ac}	word	@@@z_xor_h
	{ad}	word	@@@z_xor_l
	{ae}	word	@@@z_xor_m
	{af}	word	@@@z_xor_a

	{b0}	word	@@@z_or_b
	{b1}	word	@@@z_or_c
	{b2}	word	@@@z_or_d
	{b3}	word	@@@z_or_e
	{b4}	word	@@@z_or_h
	{b5}	word	@@@z_or_l
	{b6}	word	@@@z_or_m
	{b7}	word	@@@z_or_a

	{b8}	word	@@@z_cp_b
	{b9}	word	@@@z_cp_c
	{ba}	word	@@@z_cp_d
	{bb}	word	@@@z_cp_e
	{bc}	word	@@@z_cp_h
	{bd}	word	@@@z_cp_l
	{be}	word	@@@z_cp_m
	{bf}	word	@@@z_cp_a

	{c0}	word	@@@z_ret_nz
	{c1}	word	@@@z_pop_bc
	{c2}	word	@@@z_jp_nz
	{c3}	word	@@@z_jp
	{c4}	word	@@@z_call_nz
	{c5}	word	@@@z_push_bc
	{c6}	word	@@@z_add_a_imm8
	{c7}	word	@@@z_rst_00

	{c8}	word	@@@z_ret_z
	{c9}	word	@@@z_ret
	{ca}	word	@@@z_jp_z
	{cb}	word	@@@z_cb_xx
	{cc}	word	@@@z_call_z
	{cd}	word	@@@z_call
	{ce}	word	@@@z_adc_a_imm8
	{cf}	word	@@@z_rst_08

	{d0}	word	@@@z_ret_nc
	{d1}	word	@@@z_pop_de
	{d2}	word	@@@z_jp_nc
	{d3}	word	@@@z_out
	{d4}	word	@@@z_call_nc
	{d5}	word	@@@z_push_de
	{d6}	word	@@@z_sub_imm8
	{d7}	word	@@@z_rst_10

	{d8}	word	@@@z_ret_c
	{d9}	word	@@@z_exx
	{da}	word	@@@z_jp_c
	{db}	word	@@@z_in
	{dc}	word	@@@z_call_c
	{dd}	word	@@@z_dd_xx
	{de}	word	@@@z_sbc_a_imm8
	{df}	word	@@@z_rst_18

	{e0}	word	@@@z_ret_po
	{e1}	word	@@@z_pop_hl
	{e2}	word	@@@z_jp_po
	{e3}	word	@@@z_ex_sp_hl
	{e4}	word	@@@z_call_po
	{e5}	word	@@@z_push_hl
	{e6}	word	@@@z_and_imm8
	{e7}	word	@@@z_rst_20

	{e8}	word	@@@z_ret_pe
	{e9}	word	@@@z_jp_hl
	{ea}	word	@@@z_jp_pe
	{eb}	word	@@@z_ex_de_hl
	{ec}	word	@@@z_call_pe
	{ed}	word	@@@z_ed_xx
	{ee}	word	@@@z_xor_imm8
	{ef}	word	@@@z_rst_28

	{f0}	word	@@@z_ret_p
	{f1}	word	@@@z_pop_af
	{f2}	word	@@@z_jp_p
	{f3}	word	@@@z_di
	{f4}	word	@@@z_call_p
	{f5}	word	@@@z_push_af
	{f6}	word	@@@z_or_imm8
	{f7}	word	@@@z_rst_30

	{f8}	word	@@@z_ret_m
	{f9}	word	@@@z_ld_sp_hl
	{fa}	word	@@@z_jp_m
	{fb}	word	@@@z_ei
	{fc}	word	@@@z_call_m
	{fd}	word	@@@z_fd_xx
	{fe}	word	@@@z_cp_imm8
	{ff}	word	@@@z_rst_38

opcodes_cb
	{00}	word	@@@z_rlc_b
	{01}	word	@@@z_rlc_c
	{02}	word	@@@z_rlc_d
	{03}	word	@@@z_rlc_e
	{04}	word	@@@z_rlc_h
	{05}	word	@@@z_rlc_l
	{06}	word	@@@z_rlc_m
	{07}	word	@@@z_rlc_a

	{08}	word	@@@z_rrc_b
	{09}	word	@@@z_rrc_c
	{0a}	word	@@@z_rrc_d
	{0b}	word	@@@z_rrc_e
	{0c}	word	@@@z_rrc_h
	{0d}	word	@@@z_rrc_l
	{0e}	word	@@@z_rrc_m
	{0f}	word	@@@z_rrc_a

	{10}	word	@@@z_rl_b
	{11}	word	@@@z_rl_c
	{12}	word	@@@z_rl_d
	{13}	word	@@@z_rl_e
	{14}	word	@@@z_rl_h
	{15}	word	@@@z_rl_l
	{16}	word	@@@z_rl_m
	{17}	word	@@@z_rl_a

	{18}	word	@@@z_rr_b
	{19}	word	@@@z_rr_c
	{1a}	word	@@@z_rr_d
	{1b}	word	@@@z_rr_e
	{1c}	word	@@@z_rr_h
	{1d}	word	@@@z_rr_l
	{1e}	word	@@@z_rr_m
	{1f}	word	@@@z_rr_a

	{20}	word	@@@z_sla_b
	{21}	word	@@@z_sla_c
	{22}	word	@@@z_sla_d
	{23}	word	@@@z_sla_e
	{24}	word	@@@z_sla_h
	{25}	word	@@@z_sla_l
	{26}	word	@@@z_sla_m
	{27}	word	@@@z_sla_a

	{28}	word	@@@z_sra_b
	{29}	word	@@@z_sra_c
	{2a}	word	@@@z_sra_d
	{2b}	word	@@@z_sra_e
	{2c}	word	@@@z_sra_h
	{2d}	word	@@@z_sra_l
	{2e}	word	@@@z_sra_m
	{2f}	word	@@@z_sra_a

	{30}	word	@@@z_sli_b
	{31}	word	@@@z_sli_c
	{32}	word	@@@z_sli_d
	{33}	word	@@@z_sli_e
	{34}	word	@@@z_sli_h
	{35}	word	@@@z_sli_l
	{36}	word	@@@z_sli_m
	{37}	word	@@@z_sli_a

	{38}	word	@@@z_srl_b
	{39}	word	@@@z_srl_c
	{3a}	word	@@@z_srl_d
	{3b}	word	@@@z_srl_e
	{3c}	word	@@@z_srl_h
	{3d}	word	@@@z_srl_l
	{3e}	word	@@@z_srl_m
	{3f}	word	@@@z_srl_a

	{40}	word	@@@z_bit0_b
	{41}	word	@@@z_bit0_c
	{42}	word	@@@z_bit0_d
	{43}	word	@@@z_bit0_e
	{44}	word	@@@z_bit0_h
	{45}	word	@@@z_bit0_l
	{46}	word	@@@z_bit0_m
	{47}	word	@@@z_bit0_a

	{48}	word	@@@z_bit1_b
	{49}	word	@@@z_bit1_c
	{4a}	word	@@@z_bit1_d
	{4b}	word	@@@z_bit1_e
	{4c}	word	@@@z_bit1_h
	{4d}	word	@@@z_bit1_l
	{4e}	word	@@@z_bit1_m
	{4f}	word	@@@z_bit1_a

	{50}	word	@@@z_bit2_b
	{51}	word	@@@z_bit2_c
	{52}	word	@@@z_bit2_d
	{53}	word	@@@z_bit2_e
	{54}	word	@@@z_bit2_h
	{55}	word	@@@z_bit2_l
	{56}	word	@@@z_bit2_m
	{57}	word	@@@z_bit2_a

	{58}	word	@@@z_bit3_b
	{59}	word	@@@z_bit3_c
	{5a}	word	@@@z_bit3_d
	{5b}	word	@@@z_bit3_e
	{5c}	word	@@@z_bit3_h
	{5d}	word	@@@z_bit3_l
	{5e}	word	@@@z_bit3_m
	{5f}	word	@@@z_bit3_a

	{60}	word	@@@z_bit4_b
	{61}	word	@@@z_bit4_c
	{62}	word	@@@z_bit4_d
	{63}	word	@@@z_bit4_e
	{64}	word	@@@z_bit4_h
	{65}	word	@@@z_bit4_l
	{66}	word	@@@z_bit4_m
	{67}	word	@@@z_bit4_a

	{68}	word	@@@z_bit5_b
	{69}	word	@@@z_bit5_c
	{6a}	word	@@@z_bit5_d
	{6b}	word	@@@z_bit5_e
	{6c}	word	@@@z_bit5_h
	{6d}	word	@@@z_bit5_l
	{6e}	word	@@@z_bit5_m
	{6f}	word	@@@z_bit5_a

	{70}	word	@@@z_bit6_b
	{71}	word	@@@z_bit6_c
	{72}	word	@@@z_bit6_d
	{73}	word	@@@z_bit6_e
	{74}	word	@@@z_bit6_h
	{75}	word	@@@z_bit6_l
	{76}	word	@@@z_bit6_m
	{77}	word	@@@z_bit6_a

	{78}	word	@@@z_bit7_b
	{79}	word	@@@z_bit7_c
	{7a}	word	@@@z_bit7_d
	{7b}	word	@@@z_bit7_e
	{7c}	word	@@@z_bit7_h
	{7d}	word	@@@z_bit7_l
	{7e}	word	@@@z_bit7_m
	{7f}	word	@@@z_bit7_a

	{80}	word	@@@z_res0_b
	{81}	word	@@@z_res0_c
	{82}	word	@@@z_res0_d
	{83}	word	@@@z_res0_e
	{84}	word	@@@z_res0_h
	{85}	word	@@@z_res0_l
	{86}	word	@@@z_res0_m
	{87}	word	@@@z_res0_a

	{88}	word	@@@z_res1_b
	{89}	word	@@@z_res1_c
	{8a}	word	@@@z_res1_d
	{8b}	word	@@@z_res1_e
	{8c}	word	@@@z_res1_h
	{8d}	word	@@@z_res1_l
	{8e}	word	@@@z_res1_m
	{8f}	word	@@@z_res1_a

	{90}	word	@@@z_res2_b
	{91}	word	@@@z_res2_c
	{92}	word	@@@z_res2_d
	{93}	word	@@@z_res2_e
	{94}	word	@@@z_res2_h
	{95}	word	@@@z_res2_l
	{96}	word	@@@z_res2_m
	{97}	word	@@@z_res2_a

	{98}	word	@@@z_res3_b
	{99}	word	@@@z_res3_c
	{9a}	word	@@@z_res3_d
	{9b}	word	@@@z_res3_e
	{9c}	word	@@@z_res3_h
	{9d}	word	@@@z_res3_l
	{9e}	word	@@@z_res3_m
	{9f}	word	@@@z_res3_a

	{a0}	word	@@@z_res4_b
	{a1}	word	@@@z_res4_c
	{a2}	word	@@@z_res4_d
	{a3}	word	@@@z_res4_e
	{a4}	word	@@@z_res4_h
	{a5}	word	@@@z_res4_l
	{a6}	word	@@@z_res4_m
	{a7}	word	@@@z_res4_a

	{a8}	word	@@@z_res5_b
	{a9}	word	@@@z_res5_c
	{aa}	word	@@@z_res5_d
	{ab}	word	@@@z_res5_e
	{ac}	word	@@@z_res5_h
	{ad}	word	@@@z_res5_l
	{ae}	word	@@@z_res5_m
	{af}	word	@@@z_res5_a

	{b0}	word	@@@z_res6_b
	{b1}	word	@@@z_res6_c
	{b2}	word	@@@z_res6_d
	{b3}	word	@@@z_res6_e
	{b4}	word	@@@z_res6_h
	{b5}	word	@@@z_res6_l
	{b6}	word	@@@z_res6_m
	{b7}	word	@@@z_res6_a

	{b8}	word	@@@z_res7_b
	{b9}	word	@@@z_res7_c
	{ba}	word	@@@z_res7_d
	{bb}	word	@@@z_res7_e
	{bc}	word	@@@z_res7_h
	{bd}	word	@@@z_res7_l
	{be}	word	@@@z_res7_m
	{bf}	word	@@@z_res7_a

	{c0}	word	@@@z_set0_b
	{c1}	word	@@@z_set0_c
	{c2}	word	@@@z_set0_d
	{c3}	word	@@@z_set0_e
	{c4}	word	@@@z_set0_h
	{c5}	word	@@@z_set0_l
	{c6}	word	@@@z_set0_m
	{c7}	word	@@@z_set0_a

	{c8}	word	@@@z_set1_b
	{c9}	word	@@@z_set1_c
	{ca}	word	@@@z_set1_d
	{cb}	word	@@@z_set1_e
	{cc}	word	@@@z_set1_h
	{cd}	word	@@@z_set1_l
	{ce}	word	@@@z_set1_m
	{cf}	word	@@@z_set1_a

	{d0}	word	@@@z_set2_b
	{d1}	word	@@@z_set2_c
	{d2}	word	@@@z_set2_d
	{d3}	word	@@@z_set2_e
	{d4}	word	@@@z_set2_h
	{d5}	word	@@@z_set2_l
	{d6}	word	@@@z_set2_m
	{d7}	word	@@@z_set2_a

	{d8}	word	@@@z_set3_b
	{d9}	word	@@@z_set3_c
	{da}	word	@@@z_set3_d
	{db}	word	@@@z_set3_e
	{dc}	word	@@@z_set3_h
	{dd}	word	@@@z_set3_l
	{de}	word	@@@z_set3_m
	{df}	word	@@@z_set3_a

	{e0}	word	@@@z_set4_b
	{e1}	word	@@@z_set4_c
	{e2}	word	@@@z_set4_d
	{e3}	word	@@@z_set4_e
	{e4}	word	@@@z_set4_h
	{e5}	word	@@@z_set4_l
	{e6}	word	@@@z_set4_m
	{e7}	word	@@@z_set4_a

	{e8}	word	@@@z_set5_b
	{e9}	word	@@@z_set5_c
	{ea}	word	@@@z_set5_d
	{eb}	word	@@@z_set5_e
	{ec}	word	@@@z_set5_h
	{ed}	word	@@@z_set5_l
	{ee}	word	@@@z_set5_m
	{ef}	word	@@@z_set5_a

	{f0}	word	@@@z_set6_b
	{f1}	word	@@@z_set6_c
	{f2}	word	@@@z_set6_d
	{f3}	word	@@@z_set6_e
	{f4}	word	@@@z_set6_h
	{f5}	word	@@@z_set6_l
	{f6}	word	@@@z_set6_m
	{f7}	word	@@@z_set6_a

	{f8}	word	@@@z_set7_b
	{f9}	word	@@@z_set7_c
	{fa}	word	@@@z_set7_d
	{fb}	word	@@@z_set7_e
	{fc}	word	@@@z_set7_h
	{fd}	word	@@@z_set7_l
	{fe}	word	@@@z_set7_m
	{ff}	word	@@@z_set7_a

opcodes_ed
	{40}	word	@@@z_in_b_bc
	{41}	word	@@@z_out_bc_b
	{42}	word	@@@z_sbc_hl_bc
	{43}	word	@@@z_ld_abs16_bc
	{44}	word	@@@z_neg
	{45}	word	@@@z_retn
	{46}	word	@@@z_im_0
	{47}	word	@@@z_ld_i_a

	{48}	word	@@@z_in_c_bc
	{49}	word	@@@z_out_bc_c
	{4a}	word	@@@z_adc_hl_bc
	{4b}	word	@@@z_ld_bc_abs16
	{4c}	word	@@@z_neg		' undocumented
	{4d}	word	@@@z_reti
	{4e}	word	@@@z_im_0		' undocumented
	{4f}	word	@@@z_ld_r_a

	{50}	word	@@@z_in_d_bc
	{51}	word	@@@z_out_bc_d
	{52}	word	@@@z_sbc_hl_de
	{53}	word	@@@z_ld_abs16_de
	{54}	word	@@@z_neg		' undocumented
	{55}	word	@@@z_retn		' undocumented
	{56}	word	@@@z_im_1
	{57}	word	@@@z_ld_a_i

	{58}	word	@@@z_in_e_bc
	{59}	word	@@@z_out_bc_e
	{5a}	word	@@@z_adc_hl_de
	{5b}	word	@@@z_ld_de_abs16
	{5c}	word	@@@z_neg		' undocumented
	{5d}	word	@@@z_reti		' undocumented
	{5e}	word	@@@z_im_2
	{5f}	word	@@@z_ld_a_r

	{60}	word	@@@z_in_h_bc
	{61}	word	@@@z_out_bc_h
	{62}	word	@@@z_sbc_hl_hl
	{63}	word	@@@z_ld_abs16_hl_2
	{64}	word	@@@z_neg		' undocumented
	{65}	word	@@@z_retn		' undocumented
	{66}	word	@@@z_im_0		' undocumented
	{67}	word	@@@z_rrd_m

	{68}	word	@@@z_in_l_bc
	{69}	word	@@@z_out_bc_l
	{6a}	word	@@@z_adc_hl_hl
	{6b}	word	@@@z_ld_hl_abs16_2
	{6c}	word	@@@z_neg		' undocumented
	{6d}	word	@@@z_reti		' undocumented
	{6e}	word	@@@z_im_0		' undocumented
	{6f}	word	@@@z_rld_m

	{70}	word	@@@z_in_0_bc		' undocumented
	{71}	word	@@@z_out_bc_0		' undocumented
	{72}	word	@@@z_sbc_hl_sp
	{73}	word	@@@z_ld_abs16_sp
	{74}	word	@@@z_neg		' undocumented
	{75}	word	@@@z_retn		' undocumented
	{76}	word	@@@z_im_1		' undocumented
	{77}	word	@@@z_nop		' undocumented

	{78}	word	@@@z_in_a_bc
	{79}	word	@@@z_out_bc_a
	{7a}	word	@@@z_adc_hl_sp
	{7b}	word	@@@z_ld_sp_abs16
	{7c}	word	@@@z_neg		' undocumented
	{7d}	word	@@@z_reti		' undocumented
	{7e}	word	@@@z_im_2		' undocumented
	{7f}	word	@@@z_nop		' undocumented

	{80}	word	@@@z_bcde_cnt		' fake opcode
	{81}	word	@@@z_illegal
	{82}	word	@@@z_illegal
	{83}	word	@@@z_illegal
	{84}	word	@@@z_illegal
	{85}	word	@@@z_illegal
	{86}	word	@@@z_illegal
	{87}	word	@@@z_illegal

	{88}	word	@@@z_illegal
	{89}	word	@@@z_illegal
	{8a}	word	@@@z_illegal
	{8b}	word	@@@z_illegal
	{8c}	word	@@@z_illegal
	{8d}	word	@@@z_illegal
	{8e}	word	@@@z_illegal
	{8f}	word	@@@z_illegal

	{90}	word	@@@z_illegal
	{91}	word	@@@z_illegal
	{92}	word	@@@z_illegal
	{93}	word	@@@z_illegal
	{94}	word	@@@z_illegal
	{95}	word	@@@z_illegal
	{96}	word	@@@z_illegal
	{97}	word	@@@z_illegal

	{98}	word	@@@z_illegal
	{99}	word	@@@z_illegal
	{9a}	word	@@@z_illegal
	{9b}	word	@@@z_illegal
	{9c}	word	@@@z_illegal
	{9d}	word	@@@z_illegal
	{9e}	word	@@@z_illegal
	{9f}	word	@@@z_illegal

	{a0}	word	@@@z_ldi
	{a1}	word	@@@z_cpi
	{a2}	word	@@@z_ini
	{a3}	word	@@@z_outi
	{a4}	word	@@@z_illegal
	{a5}	word	@@@z_illegal
	{a6}	word	@@@z_illegal
	{a7}	word	@@@z_illegal

	{a8}	word	@@@z_ldd
	{a9}	word	@@@z_cpd
	{aa}	word	@@@z_ind
	{ab}	word	@@@z_outd
	{ac}	word	@@@z_illegal
	{ad}	word	@@@z_illegal
	{ae}	word	@@@z_illegal
	{af}	word	@@@z_illegal

	{b0}	word	@@@z_ldir
	{b1}	word	@@@z_cpir
	{b2}	word	@@@z_inir
	{b3}	word	@@@z_otir
	{b4}	word	@@@z_illegal
	{b5}	word	@@@z_illegal
	{b6}	word	@@@z_illegal
	{b7}	word	@@@z_illegal

	{b8}	word	@@@z_lddr
	{b9}	word	@@@z_cpdr
	{ba}	word	@@@z_indr
	{bb}	word	@@@z_otdr
	{bc}	word	@@@z_illegal
	{bd}	word	@@@z_illegal
	{be}	word	@@@z_illegal
	{bf}	word	@@@z_illegal

opcodes_xy
	{00}	word	@@@z_nop
	{01}	word	@@@z_ld_bc_imm16
	{02}	word	@@@z_ld_mem_bc_a
	{03}	word	@@@z_inc_bc
	{04}	word	@@@z_inc_b
	{05}	word	@@@z_dec_b
	{06}	word	@@@z_ld_b_imm8
	{07}	word	@@@z_rlca

	{08}	word	@@@z_ex_af_af2
	{09}	word	@@@z_add_xy_bc
	{0a}	word	@@@z_ld_a_mem_bc
	{0b}	word	@@@z_dec_bc
	{0c}	word	@@@z_inc_c
	{0d}	word	@@@z_dec_c
	{0e}	word	@@@z_ld_c_imm8
	{0f}	word	@@@z_rrca

	{10}	word	@@@z_djnz
	{11}	word	@@@z_ld_de_imm16
	{12}	word	@@@z_ld_mem_de_a
	{13}	word	@@@z_inc_de
	{14}	word	@@@z_inc_d
	{15}	word	@@@z_dec_d
	{16}	word	@@@z_ld_d_imm8
	{17}	word	@@@z_rla

	{18}	word	@@@z_jr
	{19}	word	@@@z_add_xy_de
	{1a}	word	@@@z_ld_a_mem_de
	{1b}	word	@@@z_dec_de
	{1c}	word	@@@z_inc_e
	{1d}	word	@@@z_dec_e
	{1e}	word	@@@z_ld_e_imm8
	{1f}	word	@@@z_rra

	{20}	word	@@@z_jr_nz
	{21}	word	@@@z_ld_xy_imm16
	{22}	word	@@@z_ld_abs16_xy
	{23}	word	@@@z_inc_xy
	{24}	word	@@@z_inc_hxy
	{25}	word	@@@z_dec_hxy
	{26}	word	@@@z_ld_hxy_imm8
	{27}	word	@@@z_daa

	{28}	word	@@@z_jr_z
	{29}	word	@@@z_add_xy_xy
	{2a}	word	@@@z_ld_xy_abs16
	{2b}	word	@@@z_dec_xy
	{2c}	word	@@@z_inc_lxy
	{2d}	word	@@@z_dec_lxy
	{2e}	word	@@@z_ld_lxy_imm8
	{2f}	word	@@@z_cpl

	{30}	word	@@@z_jr_nc
	{31}	word	@@@z_ld_sp_imm16
	{32}	word	@@@z_ld_abs16_a
	{33}	word	@@@z_inc_sp
	{34}	word	@@@z_inc_mxy
	{35}	word	@@@z_dec_mxy
	{36}	word	@@@z_ld_mxy_imm8
	{37}	word	@@@z_scf

	{38}	word	@@@z_jr_c
	{39}	word	@@@z_add_xy_sp
	{3a}	word	@@@z_ld_a_abs16
	{3b}	word	@@@z_dec_sp
	{3c}	word	@@@z_inc_a
	{3d}	word	@@@z_dec_a
	{3e}	word	@@@z_ld_a_imm8
	{3f}	word	@@@z_ccf

	{40}	word	@@@z_ld_b_b
	{41}	word	@@@z_ld_b_c
	{42}	word	@@@z_ld_b_d
	{43}	word	@@@z_ld_b_e
	{44}	word	@@@z_ld_b_hxy
	{45}	word	@@@z_ld_b_lxy
	{46}	word	@@@z_ld_b_mxy
	{47}	word	@@@z_ld_b_a

	{48}	word	@@@z_ld_c_b
	{49}	word	@@@z_ld_c_c
	{4a}	word	@@@z_ld_c_d
	{4b}	word	@@@z_ld_c_e
	{4c}	word	@@@z_ld_c_hxy
	{4d}	word	@@@z_ld_c_lxy
	{4e}	word	@@@z_ld_c_mxy
	{4f}	word	@@@z_ld_c_a

	{50}	word	@@@z_ld_d_b
	{51}	word	@@@z_ld_d_c
	{52}	word	@@@z_ld_d_d
	{53}	word	@@@z_ld_d_e
	{54}	word	@@@z_ld_d_hxy
	{55}	word	@@@z_ld_d_lxy
	{56}	word	@@@z_ld_d_mxy
	{57}	word	@@@z_ld_d_a

	{58}	word	@@@z_ld_e_b
	{59}	word	@@@z_ld_e_c
	{5a}	word	@@@z_ld_e_d
	{5b}	word	@@@z_ld_e_e
	{5c}	word	@@@z_ld_e_hxy
	{5d}	word	@@@z_ld_e_lxy
	{5e}	word	@@@z_ld_e_mxy
	{5f}	word	@@@z_ld_e_a

	{60}	word	@@@z_ld_hxy_b
	{61}	word	@@@z_ld_hxy_c
	{62}	word	@@@z_ld_hxy_d
	{63}	word	@@@z_ld_hxy_e
	{64}	word	@@@z_ld_hxy_hxy
	{65}	word	@@@z_ld_hxy_lxy
	{66}	word	@@@z_ld_h_mxy
	{67}	word	@@@z_ld_hxy_a

	{68}	word	@@@z_ld_lxy_b
	{69}	word	@@@z_ld_lxy_c
	{6a}	word	@@@z_ld_lxy_d
	{6b}	word	@@@z_ld_lxy_e
	{6c}	word	@@@z_ld_lxy_hxy
	{6d}	word	@@@z_ld_lxy_lxy
	{6e}	word	@@@z_ld_l_mxy
	{6f}	word	@@@z_ld_lxy_a

	{70}	word	@@@z_ld_mxy_b
	{71}	word	@@@z_ld_mxy_c
	{72}	word	@@@z_ld_mxy_d
	{73}	word	@@@z_ld_mxy_e
	{74}	word	@@@z_ld_mxy_h
	{75}	word	@@@z_ld_mxy_l
	{76}	word	@@@z_halt
	{77}	word	@@@z_ld_mxy_a

	{78}	word	@@@z_ld_a_b
	{79}	word	@@@z_ld_a_c
	{7a}	word	@@@z_ld_a_d
	{7b}	word	@@@z_ld_a_e
	{7c}	word	@@@z_ld_a_hxy
	{7d}	word	@@@z_ld_a_lxy
	{7e}	word	@@@z_ld_a_mxy
	{7f}	word	@@@z_ld_a_a

	{80}	word	@@@z_add_a_b
	{81}	word	@@@z_add_a_c
	{82}	word	@@@z_add_a_d
	{83}	word	@@@z_add_a_e
	{84}	word	@@@z_add_a_hxy
	{85}	word	@@@z_add_a_lxy
	{86}	word	@@@z_add_a_mxy
	{87}	word	@@@z_add_a_a

	{88}	word	@@@z_adc_a_b
	{89}	word	@@@z_adc_a_c
	{8a}	word	@@@z_adc_a_d
	{8b}	word	@@@z_adc_a_e
	{8c}	word	@@@z_adc_a_hxy
	{8d}	word	@@@z_adc_a_lxy
	{8e}	word	@@@z_adc_a_mxy
	{8f}	word	@@@z_adc_a_a

	{90}	word	@@@z_sub_b
	{91}	word	@@@z_sub_c
	{92}	word	@@@z_sub_d
	{93}	word	@@@z_sub_e
	{94}	word	@@@z_sub_hxy
	{95}	word	@@@z_sub_lxy
	{96}	word	@@@z_sub_mxy
	{97}	word	@@@z_sub_a

	{98}	word	@@@z_sbc_a_b
	{99}	word	@@@z_sbc_a_c
	{9a}	word	@@@z_sbc_a_d
	{9b}	word	@@@z_sbc_a_e
	{9c}	word	@@@z_sbc_a_hxy
	{9d}	word	@@@z_sbc_a_lxy
	{9e}	word	@@@z_sbc_a_mxy
	{9f}	word	@@@z_sbc_a_a

	{a0}	word	@@@z_and_b
	{a1}	word	@@@z_and_c
	{a2}	word	@@@z_and_d
	{a3}	word	@@@z_and_e
	{a4}	word	@@@z_and_hxy
	{a5}	word	@@@z_and_lxy
	{a6}	word	@@@z_and_mxy
	{a7}	word	@@@z_and_a

	{a8}	word	@@@z_xor_b
	{a9}	word	@@@z_xor_c
	{aa}	word	@@@z_xor_d
	{ab}	word	@@@z_xor_e
	{ac}	word	@@@z_xor_hxy
	{ad}	word	@@@z_xor_lxy
	{ae}	word	@@@z_xor_mxy
	{af}	word	@@@z_xor_a

	{b0}	word	@@@z_or_b
	{b1}	word	@@@z_or_c
	{b2}	word	@@@z_or_d
	{b3}	word	@@@z_or_e
	{b4}	word	@@@z_or_hxy
	{b5}	word	@@@z_or_lxy
	{b6}	word	@@@z_or_mxy
	{b7}	word	@@@z_or_a

	{b8}	word	@@@z_cp_b
	{b9}	word	@@@z_cp_c
	{ba}	word	@@@z_cp_d
	{bb}	word	@@@z_cp_e
	{bc}	word	@@@z_cp_hxy
	{bd}	word	@@@z_cp_lxy
	{be}	word	@@@z_cp_mxy
	{bf}	word	@@@z_cp_a

	{c0}	word	@@@z_ret_nz
	{c1}	word	@@@z_pop_bc
	{c2}	word	@@@z_jp_nz
	{c3}	word	@@@z_jp
	{c4}	word	@@@z_call_nz
	{c5}	word	@@@z_push_bc
	{c6}	word	@@@z_add_a_imm8
	{c7}	word	@@@z_rst_00

	{c8}	word	@@@z_ret_z
	{c9}	word	@@@z_ret
	{ca}	word	@@@z_jp_z
	{cb}	word	@@@z_xy_cb_xx
	{cc}	word	@@@z_call_z
	{cd}	word	@@@z_call
	{ce}	word	@@@z_adc_a_imm8
	{cf}	word	@@@z_rst_08

	{d0}	word	@@@z_ret_nc
	{d1}	word	@@@z_pop_de
	{d2}	word	@@@z_jp_nc
	{d3}	word	@@@z_out
	{d4}	word	@@@z_call_nc
	{d5}	word	@@@z_push_de
	{d6}	word	@@@z_sub_imm8
	{d7}	word	@@@z_rst_10

	{d8}	word	@@@z_ret_c
	{d9}	word	@@@z_exx
	{da}	word	@@@z_jp_c
	{db}	word	@@@z_in
	{dc}	word	@@@z_call_c
	{dd}	word	@@@z_dd_xx
	{de}	word	@@@z_sbc_a_imm8
	{df}	word	@@@z_rst_18

	{e0}	word	@@@z_ret_po
	{e1}	word	@@@z_pop_xy
	{e2}	word	@@@z_jp_po
	{e3}	word	@@@z_ex_sp_xy
	{e4}	word	@@@z_call_po
	{e5}	word	@@@z_push_xy
	{e6}	word	@@@z_and_imm8
	{e7}	word	@@@z_rst_20

	{e8}	word	@@@z_ret_pe
	{e9}	word	@@@z_jp_xy
	{ea}	word	@@@z_jp_pe
	{eb}	word	@@@z_ex_de_hl
	{ec}	word	@@@z_call_pe
	{ed}	word	@@@z_ed_xx
	{ee}	word	@@@z_xor_imm8
	{ef}	word	@@@z_rst_28

	{f0}	word	@@@z_ret_p
	{f1}	word	@@@z_pop_af
	{f2}	word	@@@z_jp_p
	{f3}	word	@@@z_di
	{f4}	word	@@@z_call_p
	{f5}	word	@@@z_push_af
	{f6}	word	@@@z_or_imm8
	{f7}	word	@@@z_rst_30

	{f8}	word	@@@z_ret_m
	{f9}	word	@@@z_ld_sp_xy
	{fa}	word	@@@z_jp_m
	{fb}	word	@@@z_ei
	{fc}	word	@@@z_call_m
	{fd}	word	@@@z_fd_xx
	{fe}	word	@@@z_cp_imm8
	{ff}	word	@@@z_rst_38

opcodes_xy_cb
	{00-07}	word	@@@z_rlc_mxy
	{08-0f}	word	@@@z_rrc_mxy
	{10-17}	word	@@@z_rl_mxy
	{18-1f}	word	@@@z_rr_mxy
	{20-27}	word	@@@z_sla_mxy
	{28-2f}	word	@@@z_sra_mxy
	{30-37}	word	@@@z_sli_mxy
	{38-3f}	word	@@@z_srl_mxy
	{40-47}	word	@@@z_bit0_mxy
	{48-4f}	word	@@@z_bit1_mxy
	{50-57}	word	@@@z_bit2_mxy
	{58-5f}	word	@@@z_bit3_mxy
	{60-67}	word	@@@z_bit4_mxy
	{68-6f}	word	@@@z_bit5_mxy
	{70-77}	word	@@@z_bit6_mxy
	{78-7f}	word	@@@z_bit7_mxy
	{80-87}	word	@@@z_res0_mxy
	{88-8f}	word	@@@z_res1_mxy
	{90-97}	word	@@@z_res2_mxy
	{98-9f}	word	@@@z_res3_mxy
	{a0-a7}	word	@@@z_res4_mxy
	{a8-af}	word	@@@z_res5_mxy
	{b0-b7}	word	@@@z_res6_mxy
	{b8-bf}	word	@@@z_res7_mxy
	{c0-c7}	word	@@@z_set0_mxy
	{c8-cf}	word	@@@z_set1_mxy
	{d0-d7}	word	@@@z_set2_mxy
	{d8-df}	word	@@@z_set3_mxy
	{e0-e7}	word	@@@z_set4_mxy
	{e8-ef}	word	@@@z_set5_mxy
	{f0-f7}	word	@@@z_set6_mxy
	{f8-ff}	word	@@@z_set7_mxy

#ifdef TRS80
rd_memory
	'	x000-x3ff x400-x7ff x800-xbff xc00-xfff
	word	rd_rom, rd_rom, rd_rom, rd_rom	' 0000-0fff
	word	rd_rom, rd_rom, rd_rom, rd_rom	' 1000-1fff
	word	rd_rom, rd_rom, rd_rom, rd_rom	' 2000-2fff
	word	rd_nop, rd_i_o, rd_kbd, rd_scr	' 3000-3fff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' 4000-4fff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' 5000-5fff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' 6000-6fff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' 7000-7fff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' 8000-8fff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' 9000-9fff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' a000-afff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' b000-bfff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' c000-cfff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' d000-dfff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' e000-efff
	word	rd_ram, rd_ram, rd_ram, rd_ram	' f000-ffff

wr_memory
	'	x000-x3ff x400-x7ff x800-xbff xc00-xfff
	word	wr_rom, wr_rom, wr_rom, wr_rom	' 0000-0fff
	word	wr_rom, wr_rom, wr_rom, wr_rom	' 1000-1fff
	word	wr_rom, wr_rom, wr_rom, wr_rom	' 2000-2fff
	word	wr_nop, wr_i_o, wr_nop, wr_scr	' 3000-3fff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' 4000-4fff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' 5000-5fff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' 6000-6fff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' 7000-7fff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' 8000-8fff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' 9000-9fff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' a000-afff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' b000-bfff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' c000-cfff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' d000-dfff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' e000-efff
	word	wr_ram, wr_ram, wr_ram, wr_ram	' f000-ffff
#endif TRS80
